unit fMHA_ClerkEntry;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, Grids, AdvGrid, BaseGrid;

type
  TfrmCE = class(TForm)
    stsbrCE: TStatusBar;
    pnlCE: TPanel;
    btnClose: TButton;
    grdCE: TAdvStringGrid;
    btnJump: TButton;
    dtpkrCE: TDateTimePicker;
    lblDate: TLabel;
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure grdCEGetCellColor(Sender: TObject; ARow, ACol: Integer;
      AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
    procedure FormResize(Sender: TObject);
    procedure grdCEMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure btnJumpClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure grdCECellChanging(Sender: TObject; OldRow, OldCol, NewRow,
      NewCol: Integer; var Allow: Boolean);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    MyCol, MyRow, MaxNumber: integer;
    FbtnCloseClick, FNewTest: Boolean;
    procedure InitGrid;
    procedure ResizeColWidths;
    procedure ChangeTopRow;
    procedure StartNextTest;
    procedure CalculateRowCol(AItemNumber: integer);
    function CellItemNumber(ARow: integer; ACol: integer): integer;
    procedure ShowItem;
  public
    { Public declarations }
  end;

var
  frmCE: TfrmCE;

implementation

{$R *.DFM}

uses
  TRPCB, //for RPCBroker functions
  ORfn, //for "mixedcase"
  ORNet, //for CPRS M connectivity stuff
  uCore, //for Patient
  fMHA_Main,
  uMHA_MHInstruments,
  rMHA_Vista,
  fMHA_TestEntry,
  uMHA_MsgDlg,
  uMHA_BSI,
  uMHA_DM; // Datamodule that holds Fileman components.



procedure TfrmCE.btnCloseClick(Sender: TObject);
begin

  FbtnCloseClick := True;
  TestEngine.ClerkEntryDate := DateToStr(dtpkrCE.Date);

  //save the test if complete
  if TestEngine.EndOfTest then
    TestEngine.SaveTestResults;

  //Chance to finish the test if incomplete
  if TestEngine.WantToFinishTest then
    Exit;

  //ask if the user wants to save unsaved test data
  TestEngine.UnsavedTestWarning;

  //start next test if there is one, exit if not
  if (TestList.CurrentTestID < TestList.NumberOfTests) then
    StartNextTest
  else
    close;
end;

procedure TfrmCE.FormCreate(Sender: TObject);
var r: TRect;
begin
  //set form top and height to work area (i.e., desktop size minus task bars)
  SystemParametersInfo(SPI_GetWorkArea, 0, @r, 0);
  BoundsRect := r;
  Top := r.Top;
  Height := r.Bottom - r.Top;

  //set form width to 638 and center it
  Width := 638;
  Left := (r.Right - Width) div 2;

  TestList.CurrentTestID := 0;

  //Don't allow test date to be later than date of death if patient dead
  if PatientSubSession.PatientIsDead then
  begin
    dtpkrCE.Date := PatientSubSession.DOD;
    dtpkrCE.MaxDate := PatientSubSession.DOD;
  end
  else
  begin
    dtpkrCE.Date := Date;
    dtpkrCE.MaxDate := Date;
  end;

  TestEngine.ClerkEntryDate := DateToStr(Date);

end;

procedure TfrmCE.InitGrid;
var
  i, j, RowNum1, RowNum2: integer;
begin
  with grdCE do
  begin
    Clear;
    Cells[0, 0] := 'Item No.';

    if TestEngine.ClerkEntryValidChoices.Count < 10 then
    begin
      RowCount := 2;
      ColCount := TestEngine.ClerkEntryValidChoices.Count + 1;
    end
    else
    begin
      ColCount := 11;
      //rowcount = number of items divided by 10 (integer division)
      RowCount := (TestEngine.ClerkEntryValidChoices.Count div 10) + 1;
      //increment rowcount to make room for the remainder of items
      if (TestEngine.ClerkEntryValidChoices.Count mod 10) > 0 then
        RowCount := RowCount + 1;
    end;

    //use i mod 10 so last column will be 0 instead of 10
    for i := 1 to ColCount - 1 do
      Cells[i, 0] := IntToStr(i mod 10);

    //label the first cell of each column, e.g., 1-10, 11-20, 21-22
    RowNum1 := 1;
    for i := 1 to RowCount - 1 do
    begin
      j := TestEngine.ClerkEntryValidChoices.Count - RowNum1;
      if (j > 10) then
        RowNum2 := 9
      else
        //This is necessary for the Beck scales, which have 21 items
        if (j = 0) then
          RowNum2 := 0
        else
          if ((j mod 10) = 0) then
            RowNum2 := 9
          else
            RowNum2 := (j mod 10);

      Cells[0, i] := IntToStr(RowNum1) + '-' + IntToStr(RowNum1 + RowNum2);
      RowNum1 := RowNum1 + 10
    end;

    TopRow := 1;

  end;

  ResizeColWidths;

end;

procedure TfrmCE.ResizeColWidths;
var
  icol, iwidth: integer;
begin
  iwidth := (grdCE.Width - grdCE.ColWidths[0] - 30) div 10;
  for icol := 1 to grdCE.ColCount - 1 do
    grdCE.ColWidths[icol] := iwidth;

end;

procedure TfrmCE.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if ((key = '^') and not (TestItem.ItemNumber = 1)) then
  begin
    TestItem.ItemNumber := TestItem.ItemNumber - 1;
    ShowItem;
    Exit;
  end;

  key := UpCase(key);

  //check to see that a valid response was entered
  if pos(key, TestEngine.ClerkEntryValidChoices.Strings[TestItem.ItemNumber - 1]) > 0 then
  begin
    grdCE.Cells[MyCol, MyRow] := key;
    TestEngine.AddToChoicesMade(key);
    if TestItem.ItemNumber < TestEngine.ClerkEntryValidChoices.Count then
    begin
      TestItem.ItemNumber := TestItem.ItemNumber + 1;
      ShowItem;
      stsbrCE.Color := clBtnFace;
    end;
  end
  else
    if (key in ['A'..'Z']) or (key in ['0'..'9']) then
    begin
      Beep;
      stsbrCE.Color := clInfoBk;
    end;


end;

procedure TfrmCE.CalculateRowCol(AItemNumber: integer);
begin
  //compute row and column based on item number
  MyRow := (AItemNumber div 10) + 1;
  MyCol := (AItemNumber mod 10);
  if MyCol = 0 then
  begin
    MyRow := MyRow - 1;
    MyCol := 10;
  end;


end;

procedure TfrmCE.ChangeTopRow;
var
  NumRows: integer;
begin

  NumRows := (grdCE.Height div grdCE.RowHeights[1]) - 4;

  if ((MyRow - NumRows) > 1) then
    grdCE.TopRow := MyRow - NumRows;

  if (grdCE.Height < (4 * grdCE.RowHeights[1])) then
    grdCE.TopRow := MyRow;

end;


procedure TfrmCE.ShowItem;
var
  tmpString: string;
  i, tmpItemNumber: integer;
begin

  if (MHTest.Code = 'BSI') then
  begin

    //Reset answers in cells to reflect any changes that may have been
    //made by skip logic
    for i := 1 to 21 do
    begin
      CalculateRowCol(i);
      tmpString := BSI.ReadAnswer(i);
      if (tmpString = ' ') then
        tmpString := 'NA';
      grdCE.Cells[MyCol, MyRow] := tmpString;
    end; //for i...

    if BSI.ItemIsNotApplicable(TestItem.ItemNumber) then
      Exit;
  end;

  CalculateRowCol(TestItem.ItemNumber);

  //set top row
  ChangeTopRow;

  //select cell
  grdCE.SelectRange(MyCol, MyCol, MyRow, MyRow);

  //load status bar
  stsbrCE.Panels[0].Text := '  Item Number: ' + IntToStr(TestItem.ItemNumber);

  //insert a blank space between each valid response
  tmpString := '';
  for i := 1 to length(TestEngine.ClerkEntryValidChoices.Strings[TestItem.ItemNumber - 1]) do
    tmpString := tmpString + ' '
      + copy(TestEngine.ClerkEntryValidChoices.Strings[TestItem.ItemNumber - 1], i, 1);

  stsbrCE.Panels[1].Text := '  Valid Choices: ' + tmpString;

  //enable jump button
  btnJump.Enabled := (MaxNumber > TestItem.ItemNumber);

  //keep track of the highest item number answered
  if TestItem.ItemNumber > MaxNumber then
    MaxNumber := TestItem.ItemNumber;

end;

procedure TfrmCE.grdCEGetCellColor(Sender: TObject; ARow, ACol: Integer;
  AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
begin
  if (ARow > 0) and (ACol > 0) then
    ABrush.Color := clWindow;

  //gray unneeded cells at the end of a row
  if (ARow = (grdCE.RowCount - 1)) then
    if (ACol > (TestEngine.ClerkEntryValidChoices.Count mod 10)) then
      if not ((TestEngine.ClerkEntryValidChoices.Count mod 10) = 0) then
        ABrush.Color := clBtnFace;

end;

procedure TfrmCE.FormResize(Sender: TObject);
begin
  ResizeColWidths;
  ChangeTopRow;
end;

procedure TfrmCE.grdCEMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  grdCE.SelectRange(MyCol, MyCol, MyRow, MyRow);
end;

procedure TfrmCE.btnJumpClick(Sender: TObject);
begin
  TestItem.ItemNumber := MaxNumber;
  ShowItem;
end;

procedure TfrmCE.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin

  //back up if you're not on item #1
  if ((key = vk_Back) or (key = vk_Prior))
    and not (TestItem.ItemNumber = 1) then
  begin
    TestItem.ItemNumber := TestItem.ItemNumber - 1;
    ShowItem;
  end;

  //close if escape key pressed
  if key = vk_Escape then
  begin
    Close;
    key := $00;
    Exit;
  end;

  if ((key = vk_End) or (key = vk_Next)) then
    key := vk_Right;

  if (key in [vk_F1..vk_F12]) then
    key := $00;

end;

procedure TfrmCE.grdCECellChanging(Sender: TObject; OldRow, OldCol,
  NewRow, NewCol: Integer; var Allow: Boolean);
var
  tmpNumber: integer;
begin

  if FNewTest then
    Exit;

  //Check to see if an out-of-range cell was clicked
  tmpNumber := CellItemNumber(NewRow, NewCol);

  if (MHTest.Code = 'BSI')
    and BSI.ItemIsNotApplicable(tmpNumber) then
  begin
    Allow := False;
    Beep;
    Exit;
  end;



  if tmpNumber > MaxNumber then
  begin
    Allow := False;
    Beep;
  end
  else
  begin
    TestItem.ItemNumber := tmpNumber;
    ShowItem;
  end;

end;

procedure TfrmCE.StartNextTest;
var
  i: integer;
begin
  FNewTest := True;
  FbtnCloseClick := False;
  TestEngine.InitNextTest;

  //Change form caption
  if MHTest.Name = '' then Caption := MHTest.Code
  else Caption := MHTest.Name;

  TestEngine.ClerkEntryValidChoices.Assign(rMHA_VISTA.GetClerkEntryValidResponses(MHTest.Code));

  InitGrid;

  if (TestList.CurrentTestID = TestList.NumberOfTests) then
    btnClose.Caption := '&Close'
  else
    btnClose.Caption := '&Continue';



  if TestEngine.IsRestartedTest then
  begin
    //Reload the responses into string grid
    for i := 1 to length(TestEngine.ChoicesMade) do
    begin
      TestItem.ItemNumber := i;
      CalculateRowCol(i);
      grdCE.Cells[MyCol, MyRow] := copy(TestEngine.ChoicesMade, i, 1);
    end;
    TestItem.ItemNumber := TestItem.ItemNumber + 1;
  end
  else
  begin
    MyCol := 1;
    MyRow := 1;
    TestItem.ItemNumber := 1;


  end;

  MaxNumber := TestItem.ItemNumber;

  stsbrCE.Panels[2].Text := '  Number of items: '
    + IntToStr(TestEngine.ClerkEntryValidChoices.Count);
  ShowItem;
  grdCE.SetFocus;
  FNewTest := False;

end;

procedure TfrmCE.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if (UserSession.CPRSPatientChanged = True) then
  begin
    CanClose := True;
    Exit;
  end;

  if not FbtnCloseClick then
    CanClose := not TestEngine.WantToFinishTest;
end;

procedure TfrmCE.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if (UserSession.CPRSPatientChanged = True) then
    Exit;

  if not FbtnCloseClick then
    TestEngine.UnsavedTestWarning;
end;

procedure TfrmCE.FormShow(Sender: TObject);
begin
  Application.ProcessMessages;
  StartNextTest;
end;

function TfrmCE.CellItemNumber(ARow: integer; ACol: integer): integer;
begin
  Result := ((ARow - 1) * 10) + ACol;
end;

end.

