{***************************************************************

 UNIT NAME: fMHA_TestGraph
 PACKAGE: Mental Health
 WARNING:
 DATE CREATED: 04/02/01
 SITE NAME: Salt Lake City Mental Health Informatics Section
 DEVELOPERS: CANNON/FELDMAN
 DESCRIPTION: Graphs multi-scale psychological tests
 NOTE: This unit requires Mental Health patch 67, or greater, in
       order to run.

 HISTORY  :

 Required third-party Delphi components and libraries:

****************************************************************}

unit fMHA_TestGraph;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  TeeProcs, TeEngine, Chart, ExtCtrls, StdCtrls, CheckLst, ComCtrls, Grids,
  Vawrgrid, Series, Menus, ORCtrls, AdvGrid, MFunstr, BaseGrid;

type
  TfrmTestGraph = class(TForm)
    pnlUserInput: TPanel;
    spltVertical: TSplitter;
    pnlGraphOuter: TPanel;
    chrtTestGraph: TChart;
    grpDates: TGroupBox;
    lstDates: TCheckListBox;
    grpScales: TGroupBox;
    spltHorizontal: TSplitter;
    Series1: TLineSeries;
    Series2: TLineSeries;
    Series3: TLineSeries;
    Series4: TLineSeries;
    Series5: TLineSeries;
    Series6: TLineSeries;
    Series7: TLineSeries;
    Series8: TLineSeries;
    grdScores: TAdvStringGrid;
    Series9: TLineSeries;
    Series10: TLineSeries;
    Series11: TLineSeries;
    Series12: TLineSeries;
    pnlButtons: TPanel;
    pnlGraphButtons: TPanel;
    btnSaveGraph: TButton;
    btnPrintGraph: TButton;
    btnCopyGraph: TButton;
    pnlTableButtons: TPanel;
    btnCopyTable: TButton;
    btnPrintTable: TButton;
    btnSaveTable: TButton;
    pnlCloseButton: TPanel;
    btnClose: TButton;
    lstScaleGroups: TListBox;
    PopupMenu1: TPopupMenu;
    pumnuGraph: TMenuItem;
    pumnuTable: TMenuItem;
    pumnuGraphCopy: TMenuItem;
    pumnuGraphPrint: TMenuItem;
    pumnuGraphSave: TMenuItem;
    pumnuTableCopy: TMenuItem;
    pumnuTablePrint: TMenuItem;
    pumnuTableSave: TMenuItem;
    Series13: TLineSeries;
    procedure FormCreate(Sender: TObject);
    procedure LoadDates;
    procedure LoadScales;
    procedure lstDatesClick(Sender: TObject);
    procedure GraphScores;
    procedure grdScoresClick(Sender: TObject);
    procedure lstScaleGroupsClick(Sender: TObject);
    procedure LoadScaleGroupNames;
 //   procedure GraphKScores;
    procedure grdScoresGetCellColor(Sender: TObject; ARow, ACol: Integer;
      AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
    procedure btnSaveTableClick(Sender: TObject);
    procedure btnPrintTableClick(Sender: TObject);
    procedure btnCopyTableClick(Sender: TObject);
    procedure btnSaveGraphClick(Sender: TObject);
    procedure btnPrintGraphClick(Sender: TObject);
    procedure btnCopyGraphClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure grdScoresClickCell(Sender: TObject; arow, acol: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure InitTestID;
    procedure InitCaptions;
    function MMPIDate(ListItem: string): string;
    function MMPIType(ListItem: string): string;
    procedure WriteLine(ColNum: integer; RowNum: integer; LineNum: integer);


  private
    { Private declarations }
  public
    { Public declarations }
  end;

const
  //spad is short padding with spaces, lpad is long padding
  spad = ' ';
  lpad = '     ';

var
  frmTestGraph: TfrmTestGraph;

implementation
uses uCore, ORFN,
  fMHA_MsgDlg,
  fMHA_Main, uMHA_MHInstruments, uMHA_DM, rMHA_VISTA;
{$R *.DFM}
var
  TestID: integer;
  TestName: string;
  ASIScorer: TASIScorer;


procedure TfrmTestGraph.FormCreate(Sender: TObject);
//******************************************************************************
// PURPOSE: Init public variables, call procedures to read test data, etc.
//******************************************************************************
var r: TRect;
begin
  //set form size to work area (i.e., desktop size minus task bars)
  SystemParametersInfo(SPI_GetWorkArea, 0, @r, 0);
  BoundsRect := r;
  Top := r.Top;
  Height := r.Bottom - r.Top;
  Left := r.Left;
  Width := r.Right - r.Left;

  ASIScorer := TASIScorer.Create;

  InitTestID;
  InitCaptions;
  LoadScaleGroupNames;
  LoadDates;
  LoadScales;
  grdScores.Row := 1;
  grdScores.TopRow := 1;
  lstScaleGroups.ItemIndex := 0;
  lstDates.ItemIndex := 0;
  lstDates.Checked[0] := True;
  lstDatesClick(Self);

end;

procedure TfrmTestGraph.InitCaptions;
begin
  //put caption on form
  Caption := TestName
    + ' scores for '
    + ' ' + MixedCase(Piece(Patient.Name, ',', 2))
    + ' ' + MixedCase(Piece(Patient.Name, ',', 1))
    + '  SSN: ' + Patient.SSN;

  grpDates.Caption := '  Select ' + TestName + ' Date(s)  ';

  case TestID of
    5: grpScales.Caption := '  Select Domain Score Type  ';
    11: grpScales.Caption := '  Select Norms  ';
  else
    grpScales.Caption := '  Select ' + TestName + ' Scales  ';
  end;

end;

procedure TfrmTestGraph.InitTestID;
begin
  //Init public variables for the selected test
  TestName := 'ASI';
  TestID := 5;
  if (frmMain.pgctrlMain.ActivePage = frmMain.tbshtTestResults) then
    TestName := frmMain.grdTestResults.Cells[1, frmMain.grdTestResults.Row];

  if TestName = 'MMPI2' then
    TestID := 0;
  if TestName = 'PAI' then
    TestID := 1;
  if TestName = 'NEOPI' then
    TestID := 2;
  if (TestName = 'MCMI2') or (TestName = 'MCMI3') then
    TestID := 3;
  if TestName = 'MMP2S' then
    TestID := 4;
  if TestName = 'SF36' then
    TestID := 6;
  if TestName = 'STAI' then
    TestID := 7;
  if TestName = 'BUSS' then
    TestID := 8;
  if TestName = 'BPRS' then
    TestID := 9;
  if TestName = 'MBMD' then
    TestID := 10;
  if TestName = 'BSI18' then
    TestID := 11;

  if GraphBothMMPITypes then
  begin
    TestID := 12;
    TestName := 'MMPI2-MMP2S';
  end;

end;

procedure TfrmTestGraph.WriteLine(ColNum: integer; RowNum: integer; LineNum: integer);
begin
  with grdScores do
  begin
    Cells[0, RowNum] := lpad + Piece(CompletedTest.ScaleScores[LineNum], U, 1);
    Cells[ColNum, RowNum] := Piece(CompletedTest.ScaleScores[LineNum], U, 3);
  end;
end;



procedure TfrmTestGraph.LoadScales;
//******************************************************************************
// PURPOSE: Get scale names and scores from VISTA, write to string grid
//******************************************************************************

  //This procedure is used only if MMP2S is shown along with MMPI2
  procedure MMP2S(icol: integer);
  var
    irow: integer;
  begin
    //Validity scales
    for irow := 12 to 14 do
      WriteLine(icol, irow, irow - 12);

    //Clinical scales
    for irow := 16 to 25 do
      WriteLine(icol, irow, irow - 13);

  end; //MMP2S

  procedure MMPI2(icol: integer);
  var
    irow: integer;
  begin
    with grdScores do
    begin

      //MMPI2 Validity Scales
      Cells[0, 1] := spad + lstScaleGroups.Items[0];
      //Cannot say scale requires special handling because there is no T score
      //Hence, you have to read the second instead of the third piece in Col 1
      Cells[0, 2] := lpad + Piece(CompletedTest.ScaleScores[112], U, 1);
      Cells[icol, 2] := Piece(CompletedTest.ScaleScores[112], U, 2);

      WriteLine(icol, 3, 18); //VRIN
      WriteLine(icol, 4, 17); //TRIN
      WriteLine(icol, 5, 1); //F
      WriteLine(icol, 6, 16); //Fb
      WriteLine(icol, 7, 110); //Fp
      WriteLine(icol, 8, 0); //L
      WriteLine(icol, 9, 2); //K
      WriteLine(icol, 10, 109); //S

      Cells[0, 11] := spad + lstScaleGroups.Items[1];
      for irow := 12 to 14 do
        WriteLine(icol, irow, irow - 12);

      //Clinical Scales
      Cells[0, 15] := spad + lstScaleGroups.Items[2];
      for irow := 16 to 25 do
        WriteLine(icol, irow, irow - 13);

      //Restructured Clinical (RC) Scales
      Cells[0, 26] := spad + lstScaleGroups.Items[3];
      for irow := 27 to 35 do
        WriteLine(icol, irow, irow + 91);

      //Content Scales
      Cells[0, 36] := spad + lstScaleGroups.Items[4];
      for irow := 37 to 51 do
        WriteLine(icol, irow, irow - 9);

      //Supplementary Scales
      Cells[0, 52] := spad + lstScaleGroups.Items[5];
      for irow := 53 to 55 do
        WriteLine(icol, irow, irow - 40); //A, R & Es

      WriteLine(icol, 56, 20); //Do

      for irow := 57 to 58 do
        WriteLine(icol, irow, irow - 36); //Re & Mt

      WriteLine(icol, 59, 25); //PK
      WriteLine(icol, 60, 106); //MDS
      WriteLine(icol, 61, 111); //Ho
      WriteLine(icol, 62, 19); //O-H
      WriteLine(icol, 63, 27); //MAC-R
      WriteLine(icol, 64, 108); //AAS
      WriteLine(icol, 65, 107); //APS

      for irow := 66 to 67 do
        WriteLine(icol, irow, irow - 43); //GM & GF

      //PSY-5 Scales
      Cells[0, 68] := spad + lstScaleGroups.Items[6];
      for irow := 69 to 73 do
        WriteLine(icol, irow, irow + 44);

      //Harris-Lingoes Depression Scales
      Cells[0, 74] := spad + lstScaleGroups.Items[7];
      for irow := 75 to 79 do
        WriteLine(icol, irow, irow - 32);

      //Harris-Lingoes Hysteria Scales
      Cells[0, 80] := spad + lstScaleGroups.Items[8];
      for irow := 81 to 85 do
        WriteLine(icol, irow, irow - 33);

      //Harris-Lingoes Pd Scales
      Cells[0, 86] := spad + lstScaleGroups.Items[9];
      for irow := 87 to 91 do
        WriteLine(icol, irow, irow - 34);

      //Harris-Lingoes Pa Scales
      Cells[0, 92] := spad + lstScaleGroups.Items[10];
      for irow := 93 to 95 do
        WriteLine(icol, irow, irow - 35);

      //Harris-Lingoes Schizophrenia Scales
      Cells[0, 96] := spad + lstScaleGroups.Items[11];
      for irow := 97 to 102 do
        WriteLine(icol, irow, irow - 36);

      //Harris-Lingoes Hypomania Scales
      Cells[0, 103] := spad + lstScaleGroups.Items[12];
      for irow := 104 to 107 do
        WriteLine(icol, irow, irow - 37);

      //Social Introversion Scales
      Cells[0, 108] := spad + lstScaleGroups.Items[13];
      for irow := 109 to 111 do
        WriteLine(icol, irow, irow - 38);

      //check for CNS values out of range
      if (StrToInt(grdScores.cells[icol, 2]) > 7)
        and (StrToInt(grdScores.cells[icol, 2]) < 30) then
        MsgDlg('For the MMPI2 dated ' + grdScores.cells[icol, 0]
          + ', cannot say (CNS) = '
          + grdScores.cells[icol, 2] + '.'
          + CRLF + '' + CRLF
          + 'Tests with CNS between 8 and 29 should be interpreted with caution.'
          + CRLF + '' + CRLF
          + 'The scores for this test are shown in yellow font. '
          + CRLF + '',
          DlgCaption, mtWarning, [mbOK], 0);

      if (StrToInt(grdScores.cells[icol, 2]) > 29) then
        MsgDlg('For the MMPI2 dated ' + grdScores.cells[icol, 0]
          + ', cannot say (CNS) = '
          + grdScores.cells[icol, 2] + '.'
          + CRLF + '' + CRLF
          + 'Tests with CNS of 30 or more should not be interpreted.'
               // + ''
          + CRLF + '' + CRLF
          + 'The scores for this test are shown in red '
          + 'font to indicate they are invalid.  ' + CRLF + '',
          DlgCaption, mtWarning, [mbOK], 0);

    end; //with grdScores
  end; //MMPI2

var
  i, irow, icol, iskipped: integer;
  tmpIENS, strFieldCounter, tmpString: string;
  number: single;
begin
  //load scale names and scale scores for each instance of the test
  with grdScores do
  begin

    for icol := 1 to ColCount - 1 do
    begin

      if TestName = 'ASI' then
      begin
        for irow := icol to frmMain.grdOldASI.RowCount - 1 do
          if (frmMain.grdOldASI.Cells[4, irow] = 'Yes')
            and (Cells[icol, 0] = frmMain.grdOldASI.Cells[0, irow]) then
          begin
            tmpIENS := frmMain.grdOldASI.Cells[5, irow];
            ASIScorer.EvaluationFactors.Assign(rMHA_VISTA.GetEvaluationFactors(tmpIENS));
            ASIScorer.SeverityRatings.Assign(rMHA_VISTA.GetSeverityRatings(tmpIENS));
            Break;
          end;
      end //ASI
      else
      begin //Tests
        //Set data for first instance of the test
        if GraphBothMMPITypes then
          CompletedTest.SetScaleScores(MMPIType(lstDates.Items[icol - 1]),
            MMPIDate(lstDates.Items[icol - 1]))
        else
          CompletedTest.SetScaleScores(TestName, lstDates.Items[icol - 1]);

        //Test for permission
        if CompletedTest.ScaleScores[0] = 'no priviledge' then
        begin
          MsgDlg('You do not have permission to see '
            + TestName + ' scores.', DlgCaption, mtWarning, [mbOK], 0);
          frmTestGraph.close;
          Exit;
        end;

        //Test for bad data
        if CompletedTest.ScaleScores[0] = 'no data' then
        begin
          MsgDlg('Error in reading scores for '
            + TestName + ' administered on ' + Cells[icol, 0] + '.'
            + CRLF + '' + CRLF + 'Please notify IRMS.',
            DlgCaption, mtError, [], 0);
          Continue;
        end;
      end; //Tests

      //set row count, depending on test
      case TestID of
        0: //MMPI2
          begin
            RowCount := 112;
            MMPI2(icol);
          end;

        1: //PAI
          begin
            RowCount := 65;
            //Full scale profile
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            //ICN to SOM
            for irow := 2 to 6 do
            begin
              i := irow - 2;
              Cells[0, irow] := lpad + Piece(CompletedTest.ScaleScores[i], U, 1);
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end;
            //ANX to BOR
            for irow := 2 to 8 do
            begin
              i := irow * 4;
              Cells[0, irow + 5] := lpad + Piece(CompletedTest.ScaleScores[i], U, 1);
              Cells[icol, irow + 5] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end;
            //ANT
            irow := 14;
            i := 37;
            Cells[0, irow] := lpad + Piece(CompletedTest.ScaleScores[i], U, 1);
            Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            //ALC to AGG
            for irow := 15 to 17 do
            begin
              i := irow + 26;
              Cells[0, irow] := lpad + Piece(CompletedTest.ScaleScores[i], U, 1);
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end;
            //SUI to WRM
            for irow := 18 to 23 do
            begin
              i := irow + 29;
              Cells[0, irow] := lpad + Piece(CompletedTest.ScaleScores[i], U, 1);
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end;

            //Subscales SOM to BOR (except last BOR subscale)
            i := 1;
            for irow := 24 to 55 do
            begin
              if irow mod 4 = 0 then
              begin
                Cells[0, irow] := spad + lstScaleGroups.Items[i];
                inc(i);
              end
              else
              begin
                Cells[0, irow] := lpad + Piece(CompletedTest.ScaleScores[irow - 20], U, 1);
                Cells[icol, irow] := Piece(CompletedTest.ScaleScores[irow - 20], U, 3);
              end;
            end;

            //last BOR subscale
            irow := 56;
            i := 36;
            Cells[0, irow] := lpad + Piece(CompletedTest.ScaleScores[i], U, 1);
            Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);

            //Subscales ANT & AGG
            i := 9;
            for irow := 57 to 64 do
            begin
              if ((irow - 1) mod 4 = 0) then
              begin
                Cells[0, irow] := spad + lstScaleGroups.Items[i];
                inc(i);
              end
              else
              begin
                if irow < 61 then
                  iskipped := 20
                else
                  iskipped := 18;
                Cells[0, irow] := lpad + Piece(CompletedTest.ScaleScores[irow - iskipped], U, 1);
                Cells[icol, irow] := Piece(CompletedTest.ScaleScores[irow - iskipped], U, 3);
              end;
            end;
          end; //PAI

        2: //NEOPIR
          begin
            RowCount := 42;

          //Write names of scale groups to string grid
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            for irow := 1 to 5 do
              Cells[0, irow * 7] := spad + lstScaleGroups.Items[irow];

           //Write factor names and scores
            for irow := 2 to 6 do
            begin
              i := irow + 28;
              Cells[0, irow] := lpad + Piece(CompletedTest.ScaleScores[i], U, 1);
              number := StrToFloat(Piece(CompletedTest.ScaleScores[i], U, 3));
              Cells[icol, irow] := FloatToStrF(number, ffFixed, 0, 0);
            end;

            //Facet names and scores
            for irow := 8 to 41 do
            begin
              if irow mod 7 = 0 then
                continue;

              iskipped := (irow div 7) - 1;

              i := irow - iskipped - 8;
              Cells[0, irow] := lpad + Piece(CompletedTest.ScaleScores[i], U, 1);
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end;
          end; //NEOPIR

        3: //MCMI2
          begin
            RowCount := 31;

          //Modifier Indicies
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            for irow := 2 to 4 do
              WriteLine(icol, irow, irow - 2);

          //Clinical Personality Pattern
            Cells[0, 5] := spad + lstScaleGroups.Items[1];
            for irow := 6 to 15 do
              WriteLine(icol, irow, irow - 3);

          //Severe Personality Pathology
            Cells[0, 16] := spad + lstScaleGroups.Items[2];
            for irow := 17 to 19 do
              WriteLine(icol, irow, irow - 4);

          //Clinical Syndrome
            Cells[0, 20] := spad + lstScaleGroups.Items[3];
            for irow := 21 to 26 do
              WriteLine(icol, irow, irow - 5);

          //Severe Syndrome
            Cells[0, 27] := spad + lstScaleGroups.Items[4];
            for irow := 28 to 30 do
            begin
              i := irow - 6;
              //This is necessary to keep the scale acronym upper case
              tmpString := MixedCase(Piece(CompletedTest.ScaleScores[i], U, 1));
              tmpString[2] := UpCase(tmpString[2]);
              Cells[0, irow] := lpad + tmpString;
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end; //severe syndrome
          end; //MCMI2

        4: //MMP2s by itself
          begin
            RowCount := 16;

            //Validity scales
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            for irow := 2 to 4 do
              WriteLine(icol, irow, irow - 2);

            //Clinical scales
            Cells[0, 5] := spad + lstScaleGroups.Items[1];
            for irow := 6 to 15 do
              WriteLine(icol, irow, irow - 3);
          end; //MMP2s

        5: //ASI
          begin
            RowCount := 15;

           //Write evaluation factor scores
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            if ASIScorer.EvaluationFactors[0] = '[DATA]' then
              for irow := 2 to 6 do
              begin
                i := irow - 1;
                Cells[0, irow] := lpad
                  + MixedCase(Piece(ASIScorer.EvaluationFactors[i], U, 1));
                Cells[icol, irow] := Piece(ASIScorer.EvaluationFactors[i], U, 3);
              end;

            //Write severity ratings
            Cells[0, 7] := spad + lstScaleGroups.Items[1];
            Cells[0, 8] := lpad + 'Medical';
            Cells[0, 9] := lpad + 'Employment';
            Cells[0, 10] := lpad + 'Alcohol';
            Cells[0, 11] := lpad + 'Drug';
            Cells[0, 12] := lpad + 'Legal';
            Cells[0, 13] := lpad + 'Family/Social';
            Cells[0, 14] := lpad + 'Psychiatric';
            for irow := 1 to 7 do
            begin
              case irow of
                1: strFieldCounter := '8.12'; //Medical
                2: strFieldCounter := '9.34'; //Employment
                3: strFieldCounter := '11.18'; //Alcohol
                4: strFieldCounter := '11.185'; //Drug
                5: strFieldCounter := '14.34'; //Legal
                6: strFieldCounter := '18.29'; //Fam/Soc
                7: strFieldCounter := '19.33'; //Psych

              end; //case irow

              Cells[icol, irow + 7] :=
                MHA_DataModule.FMGets1.GetField(strFieldCounter).FMDBExternal;
            end; //for irow
          end; //ASI

        6: //SF36
          begin
            RowCount := 10;

            //100-point scales
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            for irow := 2 to 9 do
            begin
              i := irow - 2;
              Cells[0, irow] := lpad + Piece(CompletedTest.ScaleScores[i], U, 1);
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end; //100-point scales
          end; //SF36

        7: //STAI
          begin
            RowCount := 4;

            //anxiety measure
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            for irow := 2 to 3 do
              WriteLine(icol, irow, irow - 2);
          end; //STAI

        8: //BUSS
          begin
            RowCount := 10;
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            for irow := 2 to 9 do
              WriteLine(icol, irow, irow - 2);
          end; //BUSS

        9: //BPRS
          begin
            RowCount := 25;

            //Items
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            for irow := 2 to 19 do
            begin
              i := irow - 2;
              Cells[0, irow] := lpad + IntToStr(i + 1) + '  '
                + MixedCase(Piece(CompletedTest.ScaleScores[i], U, 1));
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 2);
            end; //Items

            //Scales
            Cells[0, 20] := spad + lstScaleGroups.Items[1];
            for irow := 21 to 24 do
            begin
              i := irow - 3;
              Cells[0, irow] := lpad
                + MixedCase(Piece(CompletedTest.ScaleScores[i], U, 1));
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 2);
            end; //Scales

          end; //BPRS
        10: //MBMD
          begin
            RowCount := 35;

            //Psychiatric Indications
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            for irow := 2 to 6 do
            begin
              i := irow + 8;
              tmpString := Piece(CompletedTest.ScaleScores[i], U, 1);
              Cells[0, irow] := lpad + Piece(tmpString, ' ', 1) + ' '
                + MixedCase(Piece(tmpString, ' ', 2));
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end; //Psychiatric Indications

            //Coping Styles
            Cells[0, 7] := spad + lstScaleGroups.Items[1];
            for irow := 8 to 18 do
            begin
              i := irow + 7;
              tmpString := Piece(CompletedTest.ScaleScores[i], U, 1);
              Cells[0, irow] := lpad + Piece(tmpString, ' ', 1) + ' '
                + MixedCase(Piece(tmpString, ' ', 2));
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end; //Coping Styles

            //Stress Moderators
            Cells[0, 19] := spad + lstScaleGroups.Items[2];
            for irow := 20 to 25 do
            begin
              i := irow + 6;
              Cells[0, irow] := lpad
                + MixedCase(Piece(CompletedTest.ScaleScores[i], U, 1));
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end; //Stress Moderators

            //Treatment Prognostics
            Cells[0, 26] := spad + lstScaleGroups.Items[3];
            for irow := 27 to 31 do
            begin
              i := irow + 5;
              Cells[0, irow] := lpad
                + MixedCase(Piece(CompletedTest.ScaleScores[i], U, 1));
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end; //Treatment Prognostics

            //Management Guides
            Cells[0, 32] := spad + lstScaleGroups.Items[4];
            for irow := 33 to 34 do
            begin
              i := irow + 4;
              Cells[0, irow] := lpad
                + MixedCase(Piece(CompletedTest.ScaleScores[i], U, 1));
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end; //Management Guides

          end; //MBMD
        11: //BSI18
          begin
            RowCount := 11;

            //Community Norms
            Cells[0, 1] := spad + lstScaleGroups.Items[0];
            for irow := 2 to 5 do
            begin
              i := irow - 2;
              Cells[0, irow] := lpad
                + MixedCase(Piece(CompletedTest.ScaleScores[i], U, 1));
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end;

            //Oncology Norms
            Cells[0, 6] := spad + lstScaleGroups.Items[1];
            for irow := 7 to 10 do
            begin
              i := irow - 3;
              Cells[0, irow] := lpad
                + MixedCase(Piece(CompletedTest.ScaleScores[i], U, 1));
              Cells[icol, irow] := Piece(CompletedTest.ScaleScores[i], U, 3);
            end;

          end; //BSI18
        12: //both MMPI2 and MMP2S
          begin
            RowCount := 102;

            if (MMPIType(lstDates.Items[icol - 1]) = 'MMP2S') then
              MMP2S(icol)
            else
              MMPI2(icol);
          end; //12

      end; //case testid
    end; //for icol
  end; //with grd

  //set row heights
  with grdScores do
    for i := 0 to RowCount - 1 do
      RowHeights[i] := 18;

end;


procedure TfrmTestGraph.LoadDates;
//******************************************************************************
// PURPOSE: Copy test dates from the main form to list box, add dates to string
//          grid
//******************************************************************************
var
  i: integer;
begin

  lstDates.Clear;

  //copy test dates from the main form
  case TestID of
    5: //ASI
      with frmMain.grdOldASI do
        for i := 1 to RowCount - 1 do
          if (Cells[4, i] = 'Yes')
            and (Cells[3, i] = G12OK) then
            lstDates.Items.Add(Cells[0, i]);

    12: //both MMPI2 and MMP2S
      with frmMain.grdTestResults do
        for i := 1 to RowCount - 1 do
          if (Cells[1, i] = 'MMPI2')
            or (Cells[1, i] = 'MMP2S') then
            lstDates.Items.Add(Cells[0, i] + ' ' + Cells[1, i]);

  else
    with frmMain.grdTestResults do
      for i := 1 to RowCount - 1 do
        if Cells[1, i] = TestName then
          lstDates.Items.Add(Cells[0, i]);
  end; //case TestID

  //write test dates to string grid, init column number and widths
  with grdScores do
  begin

    //number of columns = number of tests plus 1
    ColCount := lstDates.Items.Count + 1;

    //set column width and label for scale name column
    ColWidths[0] := 250;
    Cells[0, 0] := ' ' + 'Scale';

    //set column width and date label for each test
    for i := 1 to ColCount - 1 do
    begin
      ColWidths[i] := 65;
      Cells[i, 0] := lstDates.Items[i - 1];
    end;
  end;

end;

function TfrmTestGraph.MMPIDate(ListItem: string): string;
begin
  result := Piece(ListItem, ' ', 1);
end;

function TfrmTestGraph.MMPIType(ListItem: string): string;
begin
  result := Piece(ListItem, ' ', 2);
end;



procedure TfrmTestGraph.lstDatesClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Calls graphing procedure when new set of tests is chosen
//******************************************************************************
begin

  with lstDates do
  begin
    if ((ItemIndex > 4) and
      (Checked[ItemIndex])) then
    begin
      MsgDlg('Only the 5 most recent tests can be graphed.',
        DlgCaption, mtInformation, [mbOK], 0);
      Checked[ItemIndex] := False;
      Exit;
    end; // if
  end; //with lstDates


  //exit if user selects ASI severity ratings for a record that doesn't have them
  if (TestName = 'ASI') and (lstScaleGroups.ItemIndex = 1) then
  begin
    if (grdScores.Cells[lstDates.ItemIndex + 1, 8] = '') then
    begin
      Beep;
      lstDates.Checked[lstDates.ItemIndex] := False;
      Exit;
    end;
  end; //ASI severity ratings check

  GraphScores;
end;


procedure TfrmTestGraph.GraphScores;
//******************************************************************************
// PURPOSE: Graph scores of selected tests
//******************************************************************************
var
  icol, irow, iseries, FirstRow, LastRow, ordinate: integer;
  MyColor: TColor;
  xlabel, CellValue: string;
  Delimiter: Char;
begin

  //Set ordinate only to remove them from warning list
  ordinate := 0;

  //Find and set first row to be graphed
  FirstRow := grdScores.Row;
  while (copy(grdScores.cells[0, FirstRow], 3, 1) = ' ') do
    dec(FirstRow);

  inc(FirstRow);

  //skip the cannot say scale on the MMPI2
  if (FirstRow = 2) and (TestID = 0) then
    FirstRow := 3;

   //Find and set last row to be graphed
  LastRow := FirstRow;
  while (copy(grdScores.cells[0, LastRow], 3, 1) = ' ') do
    inc(LastRow);
  dec(LastRow);
  if LastRow > grdScores.RowCount - 1 then
    LastRow := grdScores.RowCount - 1;
{
  //graph non-K corrected scales in a different way
  if (FirstRow = 101) and (TestID = 0) then
  begin
    GraphKScores;
    Exit;
  end;
 }
  //clear the chart
  with chrtTestGraph do
  begin


    for iseries := 0 to 12 do
      SeriesList.Series[iseries].Clear;

    for iseries := 3 to 12 do
      SeriesList.Series[iseries].Active := False;

    Legend.FirstValue := 3;
    Title.Text.Clear;
    Delimiter := ' ';

    //set left axis
    with LeftAxis do
    begin
      Title.Caption := 'T Scores';
      Increment := 10;

      case TestID of
        0, 4, 12: //MMPI2 or MMP2s or both
          begin
            Maximum := 120;
            Minimum := 30;
          end; //MMPI2
        1: //PAI
          begin
            Maximum := 100;
            Minimum := 30;
          end; //PAI
        2: //NEOPI
          begin
            Maximum := 75;
            Minimum := 25;
            Delimiter := ':';
          end; //NEOPI
        3: //MCMI2
          begin
            Maximum := 115;
            Minimum := 0;
            Title.Caption := 'BR Scores';
          end; //MCMI2
        5: //ASI
          begin
            if lstScaleGroups.ItemIndex = 0 then
            begin //Evaluation Factors
              Maximum := 75;
              Minimum := 25;
            end //Evaluation Factors
            else
            begin //Severity Ratings
              Minimum := 0;
              Maximum := 9;
              Title.Caption := 'Ratings';
              Increment := 1;
            end; //Severity Ratings
          end; //ASI
        6: //SF36
          begin
            Maximum := 100;
            Minimum := 0;
            Title.Caption := 'Scores';
          end; //SF36
        7: //STAI
          begin
            Maximum := 100;
            Minimum := 30;
          end; //STAI
        8: //BUSS
          begin
            Maximum := 100;
            Minimum := 30;
          end; //BUSS
        9: //BPRS
          begin
            if lstScaleGroups.ItemIndex = 0 then
            begin //Items
              Minimum := 0;
              Maximum := 6;
              Title.Caption := 'Ratings';
              Increment := 2;
            end //Items
            else
            begin //Scores
              Minimum := 0;
              Maximum := 18;
              Title.Caption := 'Scores';
              Increment := 3;
            end; //Scores
          end; //BPRS
        10: //MBMD
          begin
            Maximum := 100;
            Minimum := 0;
            Title.Caption := 'Prevalence Scores';
          end; //MBMD
        11: //BSI18
          begin
            Maximum := 80;
            Minimum := 30;
          end; //BSI18

      end; //case
    end; //with left axis
  end; //with chrtTestGraph

  //draw threshold lines
  for iseries := 0 to 2 do
  begin
    MyColor := clBlack;
    case iseries of
      0:
        begin
          case TestID of
            0..1, 4, 12: ordinate := 50;
            2: ordinate := 45;
            3: ordinate := 60;
            10: ordinate := 75;
            11: ordinate := 50;
          end;
        end;
      1:
        begin
          case TestID of
            0, 4, 12: ordinate := 66;
            1: ordinate := 70;
            2: ordinate := 55;
            3: ordinate := 75;
            10: ordinate := 85;
          end;
        end;
      2:
        begin
          case TestID of
            3: ordinate := 85;
            10: ordinate := 35;
          end;
        end;
    end; // case

    with chrtTestGraph do
    begin
      for irow := FirstRow to LastRow do
      begin
        xlabel := Trim(grdScores.Cells[0, irow]);
        xlabel := Piece(xlabel, Delimiter, 1); ;

        SeriesList.Series[iseries].AddXy(irow, ordinate,
          xlabel, MyColor);
      end;
    end; //with chrtTestGraph
  end; //for iseries

 //determine which columns to graph
  for icol := 0 to lstDates.Items.Count - 1 do
    if lstDates.Checked[icol] then
    begin
      //skip this column if blank
      if grdScores.Cells[icol + 1, FirstRow] = '' then
        continue;

      iseries := icol + 3;

      if iseries > 7 then
        exit;


      with chrtTestGraph do
      begin
        SeriesList.Series[iseries].Active := True;
        MyColor := SeriesList.Series[iseries].SeriesColor;

        if (FirstRow = 3) and (TestID = 0) then
          Title.Text.Text := Trim(grdScores.Cells[0, FirstRow - 2])
        else
          Title.Text.Text := Trim(grdScores.Cells[0, FirstRow - 1]);

        for irow := FirstRow to LastRow do
        begin
          SeriesList.Series[iseries].Title := lstDates.Items[icol];

          CellValue := Trim(grdScores.Cells[icol + 1, irow]);
          //drop the decimal and values to the right of the decimal, if any
          if pos('.', CellValue) > 0 then
            CellValue := Piece(CellValue, '.', 1);

          //be sure the cell contains an integer
          if (Length(CellValue) > 0) and (CellValue[1] in ['0'..'9']) then
          begin
            ordinate := StrToInt(CellValue);

            xlabel := Trim(grdScores.Cells[0, irow]);
            xlabel := Piece(xlabel, ' ', 1); ;

            SeriesList.Series[iseries].AddXy(irow, ordinate,
              xlabel, MyColor);
          end; //if statement
        end; // for irow loop
      end; // with chrtTestGraph
    end; //for icol loop

end;

{
procedure TfrmTestGraph.GraphKScores;
//******************************************************************************
// PURPOSE: For MMPI-2, graph K-corrected on non-corrected scales together
//******************************************************************************
var
  icol, irow, iseries, ordinate: integer;
  xlabel: string;
  MyColor: TColor;
begin

  //clear the chart
  with chrtTestGraph do
  begin
    for iseries := 0 to 12 do
      SeriesList.Series[iseries].Clear;

    for iseries := 3 to 12 do
      SeriesList.Series[iseries].Active := False;

    Legend.FirstValue := 3;
    Title.Text.Clear;
  end;

  //draw threshold lines at T50 and T66
  MyColor := clBlack;
  ordinate := 50;
  for iseries := 0 to 2 do
  begin
    if iseries = 1 then
      ordinate := 66;

    with chrtTestGraph do
    begin
      for irow := 12 to 21 do
      begin
        xlabel := Trim(grdScores.Cells[0, irow]);
        xlabel := Piece(xlabel, ' ', 1); ;

        SeriesList.Series[iseries].AddXy(irow, ordinate,
          xlabel, MyColor);
      end;
    end;
  end;

  //determine which columns to graph
  for icol := 0 to lstDates.Items.Count - 1 do
    if lstDates.Checked[icol] then
    begin
      iseries := icol + 3;

      if iseries > 7 then
        exit;

      with chrtTestGraph do
      begin
        SeriesList.Series[iseries].Active := True;
        SeriesList.Series[iseries + 5].Active := True;
        MyColor := SeriesList.Series[iseries].SeriesColor;

        Title.Text.Text := 'Clinical Scales (K and Non-K Corrected)';

        //graph K corrected clinical scales
        for irow := 12 to 21 do
        begin
          ordinate := StrToInt(Trim(grdScores.Cells[icol + 1, irow]));
          if pos(':', grdScores.Cells[0, irow]) = 0 then
            xlabel := Trim(grdScores.Cells[0, irow])
          else
            xlabel := Trim(Piece(grdScores.Cells[0, irow], ':', 1));

          SeriesList.Series[iseries].AddXy(irow, ordinate,
            xlabel, MyColor);

          SeriesList.Series[iseries].Title := lstDates.Items[icol] + ' K';
        end;

        //graph non-K corrected clinical scales
        for irow := 12 to 21 do
        begin
          case irow of
            12: ordinate := StrToInt(Trim(grdScores.Cells[icol + 1, 101]));
            13..14: ordinate := StrToInt(Trim(grdScores.Cells[icol + 1, irow]));
            15: ordinate := StrToInt(Trim(grdScores.Cells[icol + 1, 102]));
            16..17: ordinate := StrToInt(Trim(grdScores.Cells[icol + 1, irow]));
            18..20: ordinate := StrToInt(Trim(grdScores.Cells[icol + 1, irow + 85]));
            21: ordinate := StrToInt(Trim(grdScores.Cells[icol + 1, irow]));
          end;

          if pos(':', grdScores.Cells[0, irow]) = 0 then
            xlabel := Trim(grdScores.Cells[0, irow])
          else
            xlabel := Trim(Piece(grdScores.Cells[0, irow], ':', 1));

          SeriesList.Series[iseries + 5].AddXy(irow, ordinate,
            xlabel, MyColor);

          SeriesList.Series[iseries + 5].Title := lstDates.Items[icol] + ' NK';
        end;
      end;
    end;
end;
}

procedure TfrmTestGraph.LoadScaleGroupNames;
//******************************************************************************
// PURPOSE: Load names of groups of scales into lstScaleGroups
//******************************************************************************
begin
  with lstScaleGroups do
  begin
    Clear;
    case TestID of
      0, 12: //MMPI-2
        begin
          Items.Add('Expanded Validity Pattern');
          Items.Add('Basic Validity Pattern');
          Items.Add('Clinical Scales');
          Items.Add('Restructured Clinical (RC) Scales');
          Items.Add('Content Scales');
          Items.Add('Supplementary Scales');
          Items.Add('PSY-5 Scales');
          Items.Add('Depression (Harris-Lingoes)');
          Items.Add('Hysteria (Harris-Lingoes)');
          Items.Add('Psychopathic Deviate (Harris-Lingoes)');
          Items.Add('Paranoia (Harris-Lingoes)');
          Items.Add('Schizophrenia (Harris-Lingoes)');
          Items.Add('Hypomania (Harris-Lingoes)');
          Items.Add('Social Introversion Scales');
        end; //MMPI-2

      1: //PAI
        begin
          Items.Add('Full Scale Profile');
          Items.Add('SOM Subscales');
          Items.Add('ANX Subscales');
          Items.Add('ARD Subscales');
          Items.Add('DEP Subscales');
          Items.Add('MAN Subscales');
          Items.Add('PAR Subscales');
          Items.Add('SCZ Subscales');
          Items.Add('BOR Subscales');
          Items.Add('ANT Subscales');
          Items.Add('AGG Subscales');
        end; //PAI

      2: //NEO-PI-R
        begin
          Items.Add('Factors');
          Items.Add('Neuroticism Facets');
          Items.Add('Extraversion Facets');
          Items.Add('Openness Facets');
          Items.Add('Agreeableness Facets');
          Items.Add('Conscientiousness Facets');
        end; //NEO-PI-R

      3: //MCMI-2
        begin
          Items.Add('Modifier Indicies');
          Items.Add('Clinical Personality Pattern');
          Items.Add('Severe Personality Pathology');
          Items.Add('Clinical Syndrome');
          Items.Add('Severe Syndrome');
        end; //MCMI-2

      4: //MMP2S
        begin
          Items.Add('Validity Scales');
          Items.Add('Clinical Scales');
        end; //MMP2S

      5: //ASI
        begin
          Items.Add('Evaluation Factors');
          Items.Add('Severity Ratings');
        end; //ASI

      6: Items.Add('Scale Scores'); //SF36
      7: Items.Add('Anxiety Measures'); //STAI
      8: Items.Add('Scale Scores'); //BUSS

      9: //BPRS
        begin
          Items.Add('Items');
          Items.Add('Scores');
        end; //BPRS

      10: //MBMD
        begin
          Items.Add('Psychiatric Indications');
          Items.Add('Coping Styles');
          Items.Add('Stress Moderators');
          Items.Add('Treatment Prognostics');
          Items.Add('Management Guides');
        end; //MBMD

      11: //BSI18
        begin
          Items.Add('Community Norms');
          Items.Add('Oncology Norms');
        end;

    end; //case
  end; //with lstScaleGroups

end;


procedure TfrmTestGraph.grdScoresClick(Sender: TObject);
//******************************************************************************
// PURPOSE:
//******************************************************************************
var
  ScaleRow, StartRow: integer;
  tmpScale: string;
begin
  //  figure out the name of the set of scales (tmpScales)
  with grdScores do
  begin
    StartRow := grdScores.Row;
    if (Cells[1, StartRow] = '') then
      tmpScale := Cells[0, StartRow]
    else
      for ScaleRow := StartRow downto 1 do
      begin
        if (Cells[1, ScaleRow] = '') then
        begin
          tmpScale := Trim(Cells[0, ScaleRow]);
          Break;
        end; //if
      end; //for
  end; //with grdScores

  for ScaleRow := 0 to lstScaleGroups.Items.Count - 1 do
  begin
    if (tmpScale = lstScaleGroups.Items[ScaleRow]) then
    begin
      lstScaleGroups.ItemIndex := ScaleRow;
      Break;
    end; //if
  end; //for

  GraphScores;

end;


procedure TfrmTestGraph.lstScaleGroupsClick(Sender: TObject);
//******************************************************************************
// PURPOSE:
//******************************************************************************
var
  tmpChoice: string;
  irow, icol: integer;
begin
  irow := lstScaleGroups.ItemIndex;
  tmpChoice := lstScaleGroups.Items[irow];

  //go to appropriate row of table of scores
  for irow := 1 to grdScores.RowCount - 1 do
    if tmpChoice = Trim(grdScores.Cells[0, irow]) then
    begin
      grdScores.Row := irow;
      grdScores.TopRow := irow;
    end;

  {If ASI Severity Ratings are selected, unselect any dates that don't have
   severity rating data}
  if (TestName = 'ASI') and (lstScaleGroups.ItemIndex = 1) then
    for icol := 1 to grdScores.ColCount - 1 do
      if (grdScores.Cells[icol, 8] = '') then
        lstDates.Checked[icol - 1] := False;

end;


procedure TfrmTestGraph.grdScoresGetCellColor(Sender: TObject; ARow,
  ACol: Integer; AState: TGridDrawState; ABrush: TBrush; AFont: TFont);
//******************************************************************************
// PURPOSE: Convert scale group names to bold font
//******************************************************************************
begin

  {*in the first column (i.e., the scale name column),
  make font bold if the third character in the cell is not a blank
  (which will be true only if it is a group heading*}
  if acol < 1 then
    if copy(grdScores.cells[acol, arow], 3, 1) = ' ' then
      afont.style := []
    else
      afont.style := [fsBold];

  //change the color of the column depending on the CNS scale for the MMPI2
  if TestName = 'MMPI2' then
  begin

  //change the color of a column
    if (acol > 0) and (arow > 0) and (StrToInt(grdScores.cells[acol, 2]) > 7)
      and (StrToInt(grdScores.cells[acol, 2]) < 30) then
      afont.Color := $00009AFF;

    if (acol > 0) and (arow > 0) and (StrToInt(grdScores.cells[acol, 2]) > 29) then
      afont.color := clRed;

  end;
end;


procedure TfrmTestGraph.btnSaveTableClick(Sender: TObject);
//******************************************************************************
// PURPOSE:  Save table to Excel file
//******************************************************************************
begin
  with MHA_DataModule.dlgSave do
  begin
    FileName := 'Test Scores.xls';
    Filter := 'Excel file (*xls)|*.xls';
    if Execute then
      grdScores.SaveToXLS(MHA_DataModule.dlgSave.FileName);
  end;
end;


procedure TfrmTestGraph.btnPrintTableClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Print table
//******************************************************************************
begin
  grdScores.PrintSettings.TitleText := Caption;
  grdScores.Print;
end;


procedure TfrmTestGraph.btnCopyTableClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Copy table to Windows clipboard
//******************************************************************************
begin
  grdScores.CopyToClipBoard;
  MsgDlg('The table has been copied to the Windows clipboard.' + CRLF +
    '' + CRLF +
    'If you like, you may paste it into an open MS Excel spreadsheet.',
    DlgCaption, mtInformation, [mbOK], 0);
end;


procedure TfrmTestGraph.btnSaveGraphClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Save graph to bmp file
//******************************************************************************
begin
  with MHA_DataModule.dlgSave do
  begin
    FileName := 'Test Scores.bmp';
    Filter := 'Bitmap files (*.bmp)|*.bmp';
    if Execute then
      chrtTestGraph.SaveToBitmapFile(MHA_DataModule.dlgSave.FileName);
  end;
end;


procedure TfrmTestGraph.btnPrintGraphClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Print graph
//******************************************************************************
begin
  with chrtTestGraph do
  begin
    Foot.Text.Text := Patient.Name + '           SSN: ' + Patient.SSN;
    PrintLandscape;
    Foot.Text.Text := '';
  end;
end;


procedure TfrmTestGraph.btnCopyGraphClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Copy graph to Windows clipboard
//******************************************************************************
begin
  chrtTestGraph.CopyToClipboardBitmap;
  MsgDlg('The graph has been copied to the Windows clipboard.' + CRLF +
    '' + CRLF +
    'If you like, you may paste it into another document (e.g., an MS Word ' +
    CRLF + 'document).', DlgCaption, mtInformation, [mbOK], 0);
end;


procedure TfrmTestGraph.btnCloseClick(Sender: TObject);
//******************************************************************************
// PURPOSE: Close form
//******************************************************************************
begin
  Close;
end;

procedure TfrmTestGraph.grdScoresClickCell(Sender: TObject; arow,
  acol: Integer);
begin
  if arow = 0 then
  begin
    if acol = 0 then
      exit;

    // exit if no ASI severity rating
    if (TestName = 'ASI') and (lstScaleGroups.ItemIndex = 1)
      and (grdScores.Cells[acol, 8] = '') then
    begin
      Beep;
      Exit;
    end;

    lstDates.Checked[acol - 1] := not lstDates.Checked[acol - 1];
    lstDatesClick(Sender);
  end;

  if acol = 0 then
    grdScores.row := arow;

end;

procedure TfrmTestGraph.FormDestroy(Sender: TObject);
begin
  ASIScorer.Free;
end;

procedure TfrmTestGraph.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = vk_Escape) then
    Close;
end;

end.

