{***************************************************************

 UNIT NAME: uMHA_ASI
 PACKAGE: Mental Health
 WARNING: Per VHA Directive ... this unit should not be modified.
 DATE CREATED: May 18, 2001
 SITE NAME: Salt Lake City Mental Health Informatics Section
 DEVELOPERS: R. FELDMAN
 DESCRIPTION: Uploads to Vista choices made during ASI interview.
 NOTE: This unit requires Mental Health patch 70, or greater, in
       order to run.

 HISTORY  :

 Required third-party Delphi components and libraries:

  CPRS
  Delphi FileMan Components

****************************************************************}
unit uMHA_ASI;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, dialogs;

type
  TASI_Vista = class(TObject)
  private
    FVistAFileNumber: string;
    FResponsesSaved: BOOLEAN;
    FRecordIEN : string;
    FChoicesMade: TStrings;
    //function VistAFieldIsValid(AFieldCount: Integer; AVISTAFileNumber,
    //  ANewRecordIEN, AVistAFieldNumber, AVistAFieldValue: string): Boolean;
    function GetVistAFileNumber: string;
    //property VistAFieldName: string read FVistAFieldName write FVistAFieldName;
    //property VistAFieldNumber: string read FVistAFieldNumber write
    //FVistAFieldNumber;
    //property VistAFieldValue: string read FVistAFieldValue write
    //FVistAFieldValue;
    property VistAFileNumber: string read GetVistAFileNumber
      write FVistAFileNumber;
    //property ProblemAreaComments: TStrings read FProblemAreaComments write
    //FProblemAreaComments;
    //property IsSigned: BOOLEAN read FIsSigned write FIsSigned;
    //property ProblemAreaName: string read FProblemAreaName write
    //FProblemAreaName;
    //property DFN: string read FDFN write FDFN;
    //property DateOfInterview: string read FDateOfInterview write
    //FDateOfInterview;
    //property DateEntered: string read FDateEntered write FDateEntered;
    //property IsComplete: BOOLEAN read FIsComplete write FIsComplete;
    //property InterviewType: string read FInterviewType write FInterviewType;
    //property Interviewer: string read FInterviewer write FInterviewer;
  protected
  public
    constructor Create;
    procedure SaveToVistA(RecordIEN: string);
    procedure SaveProgressNote(RecordIEN: string);
    destructor Destroy; override;
    property ChoicesMade: TStrings read FChoicesMade write FChoicesMade;
    property ResponsesSaved: BOOLEAN read FResponsesSaved write FResponsesSaved;
    property RecordIEN: String read FRecordIEN write FRecordIEN;
  published
  end; { TASI }

var
  ASI_Vista : TASI_Vista;

implementation

uses
  fMHA_MsgDlg,
  uCore, //For core objects: patient, user, etc.
  uMHA_DM, // Container for fileMan components
  MFunStr, // For piece
  TRPCB, //for RPCBroker functions
  ORNet;


procedure TASI_Vista.SaveProgressNote(RecordIEN: string);
var
  Return : string;
begin
  with RPCBrokerV do
    begin
      ClearParameters := True;
      ClearResults := True;
      RemoteProcedure := 'YTAPI ASI PNOTE';
      Param[0].Mult['"YSIEN"'] := RecordIEN; // Add comma to end of RecordIEN
      Param[0].PType := List;
      Call;
      Return := RPCBrokerV.Results[1]; //When successful this is returned:
        // "OK ...
      if ((Return[1] = 'O') and (Return[2] = 'K')) then
        else
        MsgDlg('The ASI progress note was not written to VistA database.',
          'Mental Health Assistant', mtError, [mbOK], 0);
    end;
end;


  //******************************************************************************
  // TASIMVFile.SaveToVistA [X]USED [ ]DONE [X]TESTED
  // PURPOSE:  Upload the previous administration's local data file to VistA
  // AUTHOR: R. FELDMAN // REVIEWER:
  // LAST MODIFICATION/REVIEW DATE: 02/26/01
  //******************************************************************************


procedure TASI_Vista.SaveToVistA(RecordIEN: string);
var
  i: Integer; //For field loop counter
  Return : string; // RPC Broker results
begin
  Return := '';
  FResponsesSaved := false; // All clear toggle

  //Create a blank, new ASI VistA record and get IEN entry for .01 field
  with RPCBrokerV do
    begin
      if RecordIEN = '' then // New record: get the new RecordIEN
        begin
          ClearParameters := True;
          ClearResults := True;
          RemoteProcedure := 'YTAPI NEW ASI';
          Call;
          if Results[0] = '[DATA]' then
            RecordIEN := Results[1]
            else
            exit;
        end;

      FRecordIEN := RecordIEN; // Pass this new/old number to FASI functions
      ClearParameters := True;
      ClearResults := True;
      RemoteProcedure := 'YTAPI ASI SAVE DATA';
      Param[0].Mult['"YSIEN"'] := FRecordIEN + ','; // Add comma to end of RecordIEN
      for i := 0 to FChoicesMade.Count - 1 do
        Param[0].Mult[inttostr(i + 1)] := Piece(FChoicesMade.Strings[i], '^', 2) + '^' +
              UpperCase(Trim(Piece(FChoicesMade.Strings[i], '^', 3)));

      Param[0].PType := List;
      Call;
      Return := RPCBrokerV.Results[1]; //When successful this is returned:
        // "OK ASI SAVED XXXX," where XXXX is the RecordIEN
      if ((Return[1] = 'O') and (Return[2] = 'K')) then
        FResponsesSaved := true
        else
        MsgDlg('The ASI was not saved in the VistA database. Please try to save it again.',
          'Mental Health Assistant', mtError, [mbOK], 0);
    end; // With RPCBroker

// test
//FChoicesMade.Add(RPCBrokerV.Results[1]);
//FChoicesMade.SaveToFile('00TestValues.txt');

end;

//******************************************************************************
// TASIMVFile.VistAFieldIsValid [X]USED [X]DONE [X]TESTED
// PURPOSE: Check for, and handle any validation errors
// AUTHOR: R. FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/15/01
//******************************************************************************
 (*
function TASI_Vista.VistAFieldIsValid(AFieldCount: Integer; AVISTAFileNumber,
  ANewRecordIEN, AVistAFieldNumber, AVistAFieldValue: string): Boolean;
begin
  //Default to no errors
  result := true;
  //Set up validation call
  DM1.FMValidator1.Setup(AVISTAFileNumber, ANewRecordIEN,
    AVistAFieldNumber, AVistAFieldValue);

  //Validation call
  if not DM1.FMValidator1.Validate then
    begin
    { TODO 3 -oFeldman -cMisc : After testing, remove this and replace with non-intrusive check. }
    //    DM1.FMValidator1.DisplayErrors;

       //Error!
      if DM1.FMValidator1.Results[0] = '^' then begin
      //      //Add error to error list file
          ValErrorList.Add(VistAFieldName + '^' + AVistAFieldNumber +
            '^' + AVistAFieldValue);
            // Force a valid value into all remaining fields that failed initial validation
          if (VistAFieldNumber = '.16') // Race
            or (VistAFieldNumber = '.17') then //Religion
            DM1.FMValidator1.Results[0] := ''
          else
            DM1.FMValidator1.Results[0] := 'X';

      //If validation error occurs in a header field, then abort transmission
      //and delete record
          if AFieldCount < 8 then Result := False;
        end;
    end;

end; *)

function TASI_Vista.GetVistAFileNumber: string;
begin
  result := FVistAFileNumber;
end;

constructor TASI_Vista.Create;
begin
  inherited Create;
  VistaFileNumber := '604';
  ChoicesMade := TStringList.Create;
end;

destructor TASI_Vista.Destroy;
begin
  ChoicesMade.destroy;
  inherited Destroy;
end;


end.

