{***************************************************************

 UNIT NAME: uMHA_MHInstruments
 PACKAGE:
 WARNING:
 DATE CREATED: Sep, 2000
 SITE NAME: Salt Lake City Mental Health Informatics Section
 DEVELOPERS: FELDMAN/CANNON
 DESCRIPTION: Business objects for Mental Health Instruments.
 NOTE:

 HISTORY  :

 Required third-party Delphi components and libraries:

****************************************************************}
unit uMHA_MHInstruments;

interface

uses
  Windows,
  Messages,
  SysUtils,
  Classes,
  Graphics,
  uMHA_BSI,
  Controls;

var
  BSI: TBSI;

type
  TTestMode = (tmStaff, tmClerk, tmPatient);

type
  TTestList = class(TObject)
  private
    FCurrentTestID: Integer;
    FTestMode: TTestMode;
    FOrderedBy: string;
    FListOfIENS: TStrings;
    function GetNumberOfTests: Integer;

  protected

  public
    constructor Create;
    destructor Destroy; override;
    property CurrentTestID: Integer read FCurrentTestID write FCurrentTestID;
    property NumberOfTests: Integer read GetNumberOfTests;
    property OrderedBy: string read FOrderedBy write FOrderedBy;
    property ListOfIENS: TStrings read FListOfIENS write FListOfIENS;
    property TestMode: TTestMode read FTestMode write FTestMode;

  published

  end;

type
  TTestItem = class
  private
    FChoicesText: TStrings;
    FNumberOfChoices: Integer;
    FFile601Data: TStrings;
    FIntroductionText: TStrings;
    FQuestionText: TStrings;
    FValidChoices: string;
    FItemNumber: Integer;

  protected

  public
    property NumberOfChoices: Integer read FNumberOfChoices;
    property ItemNumber: Integer read FItemNumber write FItemNumber;
    property File601Data: TStrings read FFile601Data;
    property IntroductionText: TStrings read FIntroductionText;
    property QuestionText: TStrings read FQuestionText;
    property ChoicesText: TStrings read FChoicesText;
    property ValidChoices: string read FValidChoices;

    constructor Create;
    destructor Destroy; override;
    procedure GetItemData;
    procedure ClearData;

  published

  end;

type
  TCompletedTest = class
  private
    FScaleScores: TStrings;
  protected

  public
    procedure SetScaleScores(aCode: string; aAdate: string);
    destructor Destroy; override;
    constructor Create;
    property ScaleScores: Tstrings read FScaleScores;

  published

  end;

  TMHTest = class
  private
    FName: string;
    FDFN: string;
    FCode: string;
    FPublisher: string;
    FNumberOfItems: Integer;

  protected

  public
    property Code: string read FCode write FCode;
    property Name: string read FName write FName; //Assigned during parsing
    property DFN: string read FDFN write FDFN; //Assigned in Tests.pas
    property NumberOfItems: Integer read FNumberOfItems write FNumberOfItems;
    property Publisher: string read FPublisher; //Assigned in InitData
    procedure InitData;
    procedure ClearData;

  published

  end;

  TTestEngine = class(TObject)
  private
    FChoicesMadeSaved: Boolean;
    FChoicesMade: string;
    FIsRestartedTest: BOOLEAN;
    FTestRecoveryFileExists: BOOLEAN;
    FTestRecoveryFileName: string;
    FTestCrashDate: string;
    FClerkEntryDate: string;
    FClerkEntryValidChoices: TStrings;
//    FWantToFinishTest: BOOLEAN;
    function GetEndOfTest: Boolean;
    function GetWantToFinishTest: BOOLEAN;
  protected

  public
    constructor Create;
    destructor Destroy; override;
    procedure UnsavedTestWarning;
    property EndOfTest: Boolean read GetEndOfTest; // write FEndOfTest;
    property ChoicesMadeSaved: Boolean read FChoicesMadeSaved write
    FChoicesMadeSaved;
    property ChoicesMade: string read FChoicesMade write FChoicesMade;
    procedure AddToChoicesMade(Value: Char);
    procedure ClearChoicesMade;
    procedure SaveTestResults;
    procedure RestartIncompleteTest(Code: string);
    procedure LoadCrashFile;
    procedure InitCrashRecoveryFile;
    procedure ClearCrashFile;
    procedure InitNextTest;
    property IsRestartedTest: BOOLEAN read FIsRestartedTest write
    FIsRestartedTest;
    property TestRecoveryFileExists: BOOLEAN read FTestRecoveryFileExists write
    FTestRecoveryFileExists;
    property TestRecoveryFileName: string read FTestRecoveryFileName write
    FTestRecoveryFileName;
    property TestCrashDate: string read FTestCrashDate write FTestCrashDate;
    property ClerkEntryDate: string read FClerkEntryDate write FClerkEntryDate;
    property ClerkEntryValidChoices: TStrings read FClerkEntryValidChoices write
    FClerkEntryValidChoices;
    property WantToFinishTest: BOOLEAN read GetWantToFinishTest;

  published

  end;

type
  TASIScorer = class
  private
    FSeverityRatings: TStrings;
    FEvaluationFactors: TStrings;
  protected
  public
    constructor Create;
    destructor Destroy; override;
    property EvaluationFactors: TStrings read FEvaluationFactors write FEvaluationFactors;
    property SeverityRatings: TStrings read FSeverityRatings write FSeverityRatings;
  published
  end; { TASIScorer }


implementation

uses fMHA_Main,
  TRPCB, MFUNSTR,
  ORNet, ORFn,
  uCore, Dialogs,
  uMHA_MsgDlg, // for message dialogs
  uMHA_DM;

//******************************************************************************
//                              TestItems methods
//******************************************************************************

//******************************************************************************
// TTestItems.Create [X]USED [X]DONE [ ]TESTED
// PURPOSE: Create and init special purpose stringlists
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/20/01
//******************************************************************************

constructor TTestItem.Create;
begin
  inherited Create;
  FChoicesText := TStringList.Create;
  FIntroductionText := TStringList.Create;
  FQuestionText := TStringList.Create;
  FFile601Data := TStringList.Create;
end;

//******************************************************************************
// TTestItems.Destroy [X]USED [X]DONE [ ]TESTED
// PURPOSE: Destroy special purpose stringlists
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/20/01
//******************************************************************************

destructor TTestItem.Destroy;
begin
  FChoicesText.Free;
  FIntroductionText.Free;
  FQuestionText.Free;
  FFile601Data.Free;
  inherited Destroy;
end;

//******************************************************************************
// TTestItem.GetItemData [X]USED [X]DONE [X]TESTED
// PURPOSE: Gets the intro, question, and choices for a test item
// AUTHOR: Feldman // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 05/02/01
//******************************************************************************

procedure TTestItem.GetItemData;
const
  TestsWithCaps = ' ATQ BPRS BUSS CES CPI CRS FES FIRO GES HLOC ICL IEQ JENK M168 '
    +
    ' MATE MCMI MHLA MHLB MHLC MILL MMPR MYER RLOC SDES SII SWS VALD WAS WES ZUNG ';

  Digits: string = '0123456789';

  WordSepSet: set of Char = [#0..#64, #91..#95, #123..#137, #139,
  #145..#153, #155, #171, #187];

var
  cSection: Char;
  TempStr: string;
  ChoicesStartFlag: boolean;
  iChoiceNum: integer;

  //This function converts string to mixed case
  function SentenceCaps(const S: string): string;
  var
    I: Integer;
    First: Boolean;
  begin
    First := True;
    Result := AnsiLowerCase(S); // Convert String to lower case

    for I := 1 to Length(S) do
    begin
      if (Result[I] = '.') or (Result[I] = '?') then
        First := True;

      if First and (Result[I] in ['a'..'z']) then
      begin
        First := False;
        Result[I] := UpCase(Result[I]);
      end;

      if (Result[I] = 'i') and (Result[I - 1] = ' ')
        and ((Result[I + 1] = ' ') or (Result[I + 1] = '''')) then
        Result[I] := 'I';

    end;
  end;

  //Process parsing
  procedure DoParse;
  var
    iLineCount: Integer;
    NewChoice: Boolean;
    FirstChar: Char;

  begin

    RPCBrokerV.CreateContext('YS BROKER1');

    ChoicesStartFlag := False;
    ClearData;


    //Retrieve item data strings
    with RPCBrokerv do begin
      RemoteProcedure := 'YTAPI QUEST';
      Param[0].Mult['"CODE"'] := MHTest.Code;
      Param[0].Mult['"ITEM"'] := IntToStr(TestItem.ItemNumber);
      Param[0].PType := List;
      Call;
      File601Data.Assign(RPCBrokerV.Results);
    end;

    //Process TestItems.Strings
    for iLineCount := 0 to File601Data.Count - 1 do
    begin

      //Get section code
      cSection := CharAt(piece(File601Data[iLineCount], U, 2), 1);

      //Process TestItems.Strings according to section
      case cSection of

        //Intro text
        'I': IntroductionText.Add(Piece(File601Data[iLineCount], U, 3));


        //Question and choices text
        'T':

          begin
            //strip the blanks, use a temp variable that's easier to type
            TempStr := Trim(Piece(File601Data[iLineCount], U, 3));

            //skip blank lines
            if Length(TempStr) > 1 then
            begin

              //determine whether it is a new choice
              FirstChar := CharAt(TempStr, 1);
              if ((pos(FirstChar, Digits) > 0) and not (MHTest.Code = 'JENK'))
                or (((CharAt(TempStr, 2) = '.') and not ((MHTest.Code = 'AIMS')
                or (MHTest.Code = 'MILL'))))
                or (CharAt(TempStr, 2) = '=')
                or (CharAt(TempStr, 3) = '=') then
              begin
                ChoicesStartFlag := True;
                NewChoice := True;
              end
              else
                NewChoice := False;

              //More spaghetti:  Item 208 in the NEOPI has a line that begins
              //with a 25
              if ((Copy(TempStr, 1, 2) = '25') and (MHTest.Code = 'NEOPI')) then
              begin
                ChoicesStartFlag := False;
                NewChoice := False;
              end;

              //Add question text if it is not a choice
              //and it is not the header for the SCL9
              if not ChoicesStartFlag and
                (pos('How much were you distressed by:', TempStr) = 0) then
                QuestionText.Add(TempStr);

              //Start new choice if it is true and it is not the last line from the RLOC
              if NewChoice and (pos('1 OR 2 YOUR ANSWER =', TempStr) = 0) then
              begin
                ChoicesText.Add(TempStr);
                iChoiceNum := ChoicesText.Count - 1;
              end;

              //Continue the last choice, if necessary
              if ChoicesStartFlag and not NewChoice then
                ChoicesText.Strings[iChoiceNum] :=
                  ChoicesText.Strings[iChoiceNum] + ' ' + TempStr;
            end;

          end;

        //Valid responses text
        'R', 'A':
          begin
            FValidChoices := (Piece(File601Data[iLineCount], U, 3));
            FNumberOfChoices := Length(ValidChoices);
          end;
      end;
    end;
  end;

  // Beginning of GetItemData method
begin

  DoParse;

  //Change the case of the Intro text
  if pos(MHTest.Code, TestsWithCaps) > 0 then
    IntroductionText.Text := SentenceCaps(IntroductionText.Text);

  //Clean up the intro text
  IntroductionText.Text := StringReplace(IntroductionText.Text, #$D#$A, ' ',
    [rfReplaceAll, rfIgnoreCase]);

  //Clean up the question text
  TempStr := StringReplace(TestItem.QuestionText.Text, #$D#$A, ' ',
    [rfReplaceAll, rfIgnoreCase]);

  //Change the case of the question text
  if pos(MHTest.Code, TestsWithCaps) > 0 then
    TempStr := SentenceCaps(TempStr);

  //Add to stringlist
  TestItem.QuestionText.Clear;
  TestItem.QuestionText.Add(TempStr);

  //Change the case of the choices text
  if (pos(MHTest.Code, TestsWithCaps) > 0) or (MHTest.Code = 'BAI') then
    for iChoiceNum := 0 to ChoicesText.Count - 1 do
      ChoicesText.Strings[iChoiceNum] :=
        SentenceCaps(ChoicesText.Strings[iChoiceNum]);

  //Fix choices for the DOM81 and DOM82
  if ((MHTest.Code = 'DOM81') or (MHTest.Code = 'DOM82'))
    and not (CharAt(ChoicesText.Text, 1) = '1') then
  begin
    ChoicesText.Clear;
    ChoicesText.Add('Yes');
    ChoicesText.Add('No');
  end;

end;

//******************************************************************************
// TTestItems.ClearData
// PURPOSE:
// AUTHOR: Feldman // REVIEWER: Feldman
// LAST REVIEW/MODIFICATION DATE: 00/00/00 // STATUS:
//******************************************************************************

procedure TTestItem.ClearData;
begin
  ChoicesText.Clear;
  FNumberOfChoices := 0; //No need to clear, because property is read every time.
  File601Data.Clear;
  IntroductionText.Clear;
  QuestionText.Clear;
  FValidChoices := ''; // No need to clear, because property is read every time.
end;

//******************************************************************************
//                            CompletedTest methods
//******************************************************************************

//******************************************************************************
// TCompletedTest.Create [X]USED [X]DONE [ ]TESTED
// PURPOSE:
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 04/03/01
//******************************************************************************

constructor TCompletedTest.Create;
begin
  inherited Create;
  FScaleScores := TStringList.Create;
end;

//******************************************************************************
// TCompletedTest.Destroy [X]USED [X]DONE [ ]TESTED
// PURPOSE:
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 04/03/01
//******************************************************************************

destructor TCompletedTest.Destroy;
begin
  FScaleScores.Free;
  inherited Destroy;
end;

//******************************************************************************
// TCompletedTest.SetScaleScores [X]USED [X]DONE [X]TESTED
// PURPOSE: Load Scalescores TStrings, given passed parameters.
// AUTHOR: FELDMAN // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 05/07/01
//******************************************************************************
//[NOTE] This is not a setter for the property, because it's passing parameters
//that cause a "incompatible types" error. This approach works well.

procedure TCompletedTest.SetScaleScores(aCode: string; aAdate: string);
var
  i: integer;
begin
  with RPCBrokerV do begin
    ClearResults := True;
    ClearParameters := True;
    CreateContext('YS BROKER1');
    RemoteProcedure := 'YTAPI SCOREIT';
    Param[0].Mult['"DFN"'] := Patient.DFN;
    Param[0].Mult['"CODE"'] := aCode;
    Param[0].Mult['"ADATE"'] := aADate;
    Param[0].PType := List;

    call;

    FScaleScores.Clear;

    //Check to see if YTAPI SCOREIT returned data and if user
    //has permission to see scale scores
    if Results[2] = 'no priviledge' then
    begin
      FScaleScores.Add('no priviledge');
      exit;
    end;

    if Results[0] <> '[DATA]' then
    begin
      FScaleScores.Add('no data');
      exit;
    end;

    //delete the header and item responses
    for i := 4 downto 0 do
      Results.Delete(i);

    for i := 0 to Results.Count - 1 do
      FScaleScores.Add(Piece(Results[i], U, 2)
        + U + Piece(Results[i], U, 3)
        + U + Piece(Results[i], U, 4));

  end;
end;

//******************************************************************************
//                            MHTest methods
//******************************************************************************

//******************************************************************************
// TMHTest.InitData [X]USED [X]DONE [X]TESTED
// PURPOSE: Initialize some TMHTest object members
// AUTHOR: Feldman// REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 08/21/01
//******************************************************************************

procedure TMHTest.InitData;
begin

  //Get DFN
  FDFN := Piece(TestList.ListOfIENS[TestList.CurrentTestID - 1], U, 1);

  //Get the test code, using FCode, to keep it read-only.
  FCode := Piece(TestList.ListOfIENS[TestList.CurrentTestID - 1], U, 2);

  if (FCode = 'BSI') then
  begin
    if (BSI = nil) then
      BSI := TBSI.Create
    else
      BSI.Clear;
  end;

  //Get the test name
  with MHA_DataModule.FMGets1 do begin
    FieldNumbers.Clear;
    RPCBroker := RPCBrokerv;
    FileNumber := '601'; //MH Package
    FieldNumbers.Add('13'); // Test name
    IENS := FDFN; //Test IENS
    GetData;
    FName := GetField('13').FMDBExternal;
  end;

  //Get number of items in a test
  with MHA_DataModule.FMGets1 do begin
    FieldNumbers.Clear;
    RPCBroker := RPCBrokerv;
    FileNumber := '601'; //MH Package
    FieldNumbers.Add('22'); // Number of items in test
    IENS := FDFN; //Test IENS
    GetData;
    FNumberOfItems := StrToInt(GetField('22').FMDBExternal);
  end;

  //Determine test publisher
  with MHA_DataModule.FMGets1 do begin
    FieldNumbers.Clear;
    RPCBroker := RPCBrokerv;
    FileNumber := '601'; //MH Package
    FieldNumbers.Add('25'); //
    IENS := FDFN; //Test IENS
    GetData;
    FPublisher := GetField('25').FMDBExternal;
  end;

  //Compensate for extra 2 fields added later. Total is actually 177
  if (MHTest.Code = 'MCMI2') or (MHTest.Code = 'MCMI3') then
    MHTest.NumberOfItems := 175;

end;

//******************************************************************************
// TMHTest.ClearData [X]USED [X]DONE [X]TESTED
// PURPOSE: Clear MHTest properties
// AUTHOR: Feldman // REVIEWER:  Cannon
// LAST MODIFICATION/REVIEW DATE: 05/08/01
//******************************************************************************

procedure TMHTest.ClearData;
begin
  if (BSI <> nil) then
    FreeAndNil(BSI);

  FName := '';
  FDFN := '';
  FCode := '';
  FNumberOfItems := 0; //This was set by InitData method
end;

//******************************************************************************
//                            TestEngine methods
//******************************************************************************

//******************************************************************************
// TTestEngine.Create [X]USED [ ]DONE [ ]TESTED
// PURPOSE: Needed to initialize TStrings properties.
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 08/16/2001
//******************************************************************************

constructor TTestEngine.Create;
begin
  inherited Create;
  FClerkEntryValidChoices := TStringList.Create;
end;


//******************************************************************************
// TTestEngine.Destroy [X]USED [ ]DONE [ ]TESTED
// PURPOSE: Needed to initialize TStrings properties.
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 08/16/2001
//******************************************************************************

destructor TTestEngine.Destroy;
begin
  if (BSI <> nil) then
    FreeAndNil(BSI);

  FClerkEntryValidChoices.free;
  inherited Destroy;
end;

//******************************************************************************
// TTestEngine.AddToChoicesMade [X]USED [X]DONE [ ]TESTED
// PURPOSE: Build user's response string
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 08/14/01
//******************************************************************************

procedure TTestEngine.AddToChoicesMade(Value: Char);
var
  PosInString, iEndPos, i: Integer;
begin

  PosInString := TestItem.ItemNumber;
  if PosInString > MHTest.NumberOfItems then
    PosInString := MHTest.NumberOfItems;
  iEndPos := length(FChoicesMade) + 1;


  //Replace a previous entry into the ChoicesMade
  if (iEndPos > PosInString) then
  begin
    Delete(FChoicesMade, PosInString, 1);
    Insert(Value, FChoicesMade, PosInString);
  end
  else
    //Add a new entry into ChoicesMade
    Insert(Value, FChoicesMade, PosInString);

  if (MHTest.Code = 'BSI') then
  begin
    if (CurrentEntryMode = emStaff) then
      BSI.WriteAnswerText(TestItem.ItemNumber,
        TestItem.ChoicesText[StrToIntDef(Value, 0)]);

    BSI.ApplySkipLogic;
  end;

  //Save current test responses state to crash-recovery file
  ClearCrashFile;

  InitCrashRecoveryFile;

  WriteLn(CrashedTestRecoveryFile,
    '** MENTAL HEALTH ASSISTANT TEST RECOVERY FILE ' +
    DateTimeToStr(Now)
    + ' **');
  WriteLn(CrashedTestRecoveryFile, RPCBrokerV.Server);
  WriteLn(CrashedTestRecoveryFile, DateToStr(date));
  WriteLn(CrashedTestRecoveryFile, TestOrderer.DUZ);
  WriteLn(CrashedTestRecoveryFile, Patient.DFN);
  WriteLn(CrashedTestRecoveryFile, MHTest.Code);
  WriteLn(CrashedTestRecoveryFile, MHTest.NumberOfItems);
  WriteLn(CrashedTestRecoveryFile, FChoicesMade);
  Flush(CrashedTestRecoveryFile);

end;

//******************************************************************************
// TTestEngine.ClearChoicesMade [X]USED [X]DONE [ ]TESTED
// PURPOSE: Clear Response String
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/20/01
//******************************************************************************

procedure TTestEngine.ClearChoicesMade;
begin
  FChoicesMade := '';
end;

//******************************************************************************
// TTestEngine.GetEndOfTest [X]USED [X]DONE [X]TESTED
// PURPOSE: Determine when a test is completed
// AUTHOR: FELDMAN // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 05/07/01
//******************************************************************************

function TTestEngine.GetEndOfTest: Boolean;
begin

  if (MHTest.Code = 'MCMI2') or (MHTest.Code = 'MCMI3') then
  begin
    if (length(FChoicesMade) >= MHTest.NumberOfItems)
      and (MHTest.NumberOfItems > 0) then
      Result := True
    else
      Result := False;

    Exit;
  end; //MCMI2


  if (length(FChoicesMade) = MHTest.NumberOfItems)
    and (MHTest.NumberOfItems > 0) then
    Result := True
  else
    Result := False;
end;

//******************************************************************************
// TMHTest.SaveTestResults [X]USED [X]DONE [X]TESTED
// PURPOSE: Save the resulting response string to the M server
// AUTHOR: R. FELDMAN // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 05/11/01
//******************************************************************************

procedure TTestEngine.SaveTestResults;
var
  tmpLength: integer;
begin


  with RPCBrokerv do
  begin
    ClearParameters := True;
    ClearResults := True;
    CreateContext('YS BROKER1');
    if TestEngine.EndOfTest then
    begin
      //for completed test

      //add clinician answers to choices made
      if (MHTest.Code = 'MCMI2') or (MHTest.Code = 'MCMI3') then
        TestEngine.ChoicesMade := TestEngine.ChoicesMade
          + PatientSubSession.MCMI2_PatientTraits;


      RemoteProcedure := 'YTAPI SAVEIT';
      if TestEngine.IsRestartedTest then
        Param[0].Mult['"STAFF"'] := TestOrderer.FirstOrderer
      else
        Param[0].Mult['"STAFF"'] := IntToStr(TestOrderer.DUZ);

      if testlist.TestMode = tmClerk then
        Param[0].Mult['"ADATE"'] := TestEngine.ClerkEntryDate
      else
        Param[0].Mult['"ADATE"'] := 'NOW';

    end
    else
    begin
      //for incomplete test

      //pad choices made for the MCMI2 and then add the clinician answers
      if (MHTest.Code = 'MCMI2') or (MHTest.Code = 'MCMI3') then
      begin
        for tmpLength := length(TestEngine.ChoicesMade) + 1 to 175 do
          TestEngine.ChoicesMade := TestEngine.ChoicesMade + ' ';

        TestEngine.ChoicesMade := TestEngine.ChoicesMade
          + PatientSubSession.MCMI2_PatientTraits;
      end;

      RemoteProcedure := 'YTAPI SAVE INCOMPLETES';
      Param[0].Mult['"NEXT"'] := IntToStr(Length(TestEngine.ChoicesMade) + 1);

      if TestRecoveryFileExists then
        Param[0].Mult['"ADATE"'] := TestCrashDate
      else
        if testlist.TestMode = tmClerk then
          Param[0].Mult['"ADATE"'] := TestEngine.ClerkEntryDate
        else
          Param[0].Mult['"ADATE"'] := 'NOW';


      if TestEngine.IsRestartedTest then
        Param[0].Mult['"ORDERBY"'] := TestOrderer.FirstOrderer
      else
        Param[0].Mult['"ORDERBY"'] := IntToStr(TestOrderer.DUZ);
    end;
    //for both complete and incomplete tests
    Param[0].Mult['"DFN"'] := Patient.DFN;
    Param[0].Mult['"CODE"'] := MHTest.CODE;
    Param[0].Mult['"R1"'] := Copy(TestEngine.ChoicesMade, 1, 200);
    Param[0].Mult['"R2"'] := Copy(TestEngine.ChoicesMade, 201, 200);
    Param[0].Mult['"R3"'] := Copy(TestEngine.ChoicesMade, 401, 200);
    Param[0].PType := List;

    try

      Call;
    except
      on EBrokerError do
      begin
        Beep;
        ShowMessage('Test results cannot be saved.  MHA will close now.');
        frmMain.Close;
        Exit;
      end;
    end;

    if (Results[0] = '[DATA]') and (Results[1] = 'saved ok') then
    begin

      //Send Vista email to orderer.
      if (user.DUZ <> TestOrderer.DUZ) then
        with RPCBrokerV do begin
          ClearParameters := true;
          ClearResults := true;
          RemoteProcedure := 'YTAPI TEST BULLETIN';
          if TestEngine.IsRestartedTest then
            Param[0].Mult['"ORD"'] := TestOrderer.FirstOrderer
          else
            Param[0].Mult['"ORD"'] := IntToStr(TestOrderer.DUZ);
          Param[0].Mult['"DFN"'] := patient.DFN; //'10121927'; //patient.DFN
          Param[0].Mult['"TEST1"'] := MHTest.Code; // 'MMPI2';
          Param[0].PType := List;
          Call;
        end;

      TestEngine.ChoicesMadeSaved := True;
    end
    else
      TestEngine.ChoicesMadeSaved := False;

  end; // with RPCBroker

  TestEngine.ClearCrashFile;

end;
//******************************************************************************
// TTestEngine.InitNextTest [X]USED []DONE []TESTED
// PURPOSE: Initialize test information with data from Vista.
// AUTHOR: Feldman // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 08/10/01
//******************************************************************************

procedure TTestEngine.InitNextTest;
begin
  //Increment the current test ID
  TestList.CurrentTestID := TestList.CurrentTestID + 1;

  //Init test engine and test item properties for new test
  MHTest.ClearData;
  TestEngine.ChoicesMadeSaved := False;
  TestEngine.ClearChoicesMade;
  //?  TestEngine.InitCrashRecoveryFile;
  TestItem.ItemNumber := 0;
  TestEngine.ClerkEntryValidChoices.Clear;

  //Check for incomplete test (indicated by '*')
  if pos('*', TestList.ListOfIENS[TestList.CurrentTestID - 1]) > 0 then
  begin
    TestEngine.IsRestartedTest := True;
    //Remove '*' from test code, e.g., CAGE* becomes CAGE
    TestList.ListOfIENS[TestList.CurrentTestID - 1] :=
      copy(TestList.ListOfIENS[TestList.CurrentTestID - 1], 1,
      length(TestList.ListOfIENS[TestList.CurrentTestID - 1]) - 1);
    TestEngine.RestartIncompleteTest(Piece(TestList.ListOfIENS[TestList.CurrentTestID - 1], U, 2));
  end
  else
    TestEngine.IsRestartedTest := False;

  //Get test name, code, publisher, and number of items
  MHTest.InitData;

end;

procedure TTestEngine.ClearCrashFile;
begin
  if FileExists(TestRecoveryFileName) then
  begin
    //Close and delete crash-recovery file
    CloseFile(CrashedTestRecoveryFile);
    Erase(CrashedTestRecoveryFile);
  end;
end;

//******************************************************************************
//                       TestList methods
//******************************************************************************

//******************************************************************************
// TTestList.Create [X]USED [X]DONE [ ]TESTED
// PURPOSE:
// AUTHOR: FELDMAN/CANNON // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/16/01
//******************************************************************************

constructor TTestList.Create;
begin
  inherited Create;
  FListOfIENS := TStringList.Create;
end;

//******************************************************************************
// TTestList.Destroy [X]USED [X]DONE [ ]TESTED
// PURPOSE:
// AUTHOR: FELDMAN/CANNON // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/16/01
//******************************************************************************

destructor TTestList.Destroy;
begin
  FListOfIENS.Free;
  inherited Destroy;
end;

//******************************************************************************
// TTestList.GetNumberOfTests [X]USED [X]DONE [X]TESTED
// PURPOSE: Get Number Of Tests
// AUTHOR: FELDMAN // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 05/07/01
//******************************************************************************
// Needs no setter

function TTestList.GetNumberOfTests: Integer;
begin
  Result := FListOfIENS.Count;
end;

//******************************************************************************
// TTestEngine.RestartIncompleteTest []USED []DONE []TESTED
// PURPOSE: Load values from a previously-saved incomplete test
// AUTHOR: Cannon // REVIEWER:  Feldman
// LAST MODIFICATION/REVIEW DATE: 05/10/01
//******************************************************************************

procedure TTestEngine.RestartIncompleteTest(Code: string);
var
  tmpIENS: string;
  tmpPos: integer;
begin
  //Get saved data for the incomplete test
  with RPCBrokerV do begin
    ClearResults := True;
    ClearParameters := True;
    CreateContext('YS BROKER1');
    RemoteProcedure := 'YTAPI GET INCOMPLETE';
    Param[0].Mult['"DFN"'] := Patient.DFN;
    Param[0].Mult['"CODE"'] := Code;
    Param[0].PType := List;

    call;

    if not (Results[0] = '[DATA]') then
    begin
      MsgDlg('Unable to retrieve results of saved incomplete test.',
        DlgCaption, mtInformation, [mbOK], 0);
      exit;
    end;

    tmpIENS := Piece(Results[1], U, 1);
    TestOrderer.FirstOrderer := Piece(Results[1], U, 7);
    TestEngine.ChoicesMade := Results[2] + Results[3] + Results[4];
  end;

  //do what you've got to do if it's an MCMI2
  if (tmpIENS = '201') or (tmpIENS = '246') then
  begin
    PatientSubSession.MCMI2_PatientTraits := copy(TestEngine.ChoicesMade, 176,
      2);
    tmpPos := pos(' ', TestEngine.ChoicesMade);
    TestEngine.ChoicesMade := copy(TestEngine.ChoicesMade, 1, tmpPos - 1);
  end;

end;

procedure TTestEngine.InitCrashRecoveryFile;
begin
  AssignFile(CrashedTestRecoveryFile, TestRecoveryFileName);
{$I-}
  ReWrite(CrashedTestRecoveryFile);
{$I+}
  if not IOResult = 0 then //begin
    MsgDlg('Test recovery file create error',
      DlgCaption, mtError, [mbOk], 0);

end;

procedure TTestEngine.LoadCrashFile;
var
  tempServerName,
    tempCrashDate,
    tempTestOrderer,
    tempPatientDFN,
    tempTestCode,
    tempNumberOfItems,
    tempChoicesMade: string;
begin
  AssignFile(CrashedTestRecoveryFile, TestRecoveryFileName);
{$I-}
  Reset(CrashedTestRecoveryFile);
{$I+}
  if not IOResult = 0 then //begin
    MsgDlg('Test recovery file open error',
      DlgCaption, mtWarning, [mbOk], 0);

  Readln(CrashedTestRecoveryFile); //Skip header
  Readln(CrashedTestRecoveryFile, tempServerName);

  if tempServerName = '' then begin
    ClearCrashFile;
    exit;
  end;

  //Check for wrong server.
  if tempServerName <> RPCBrokerV.Server then exit;

  Readln(CrashedTestRecoveryFile, tempCrashDate);
  TestCrashDate := tempCrashDate;
  Readln(CrashedTestRecoveryFile, tempTestOrderer);
  TestOrderer.FirstOrderer := tempTestOrderer;

  Readln(CrashedTestRecoveryFile, tempPatientDFN);
  patient.DFN := tempPatientDFN;
  Readln(CrashedTestRecoveryFile, tempTestCode);
  MHTest.Code := tempTestCode;
  Readln(CrashedTestRecoveryFile, tempNumberOfItems);
  MHTest.NumberOfItems := StrToInt(tempNumberOfItems);
  Readln(CrashedTestRecoveryFile, tempChoicesMade);
  ChoicesMade := tempChoicesMade;

  IsRestartedTest := true;

  ClearCrashFile;

  if Length(ChoicesMade) <> 0 then SaveTestResults;

end;

//******************************************************************************
//                       TASIScorer methods
//******************************************************************************

constructor TASIScorer.Create;
begin
  inherited Create;
  FSeverityRatings := TStringList.Create;
  FEvaluationFactors := TStringList.Create;
end;

destructor TASIScorer.Destroy;
begin
  FEvaluationFactors.free;
  FSeverityRatings.free;
  inherited Destroy;
end;

procedure TTestEngine.UnsavedTestWarning;
begin
  if not (TestList.TestMode = tmPatient) then
    //Warn if all answers to the current test have not been saved
    if not (TestEngine.ChoicesMadeSaved) and
      (length(TestEngine.ChoicesMade) > 0) then
      if MsgDlg(MHTest.Code + ' answers have not been saved to VISTA.'
        + CRLF + '' + CRLF + 'Do you want to save those answers now?',
        DlgCaption, mtWarning, [mbYes, mbNo], 0) = mrYes then
        TestEngine.SaveTestResults
      else
      begin
        TestEngine.ClearCrashFile;
        //even though the user opts not save choices made, set this
        //boolean to true the user doesn't get asked twice on form close
        //if he/she clicked on btnNextTest
        TestEngine.ChoicesMadeSaved := True;
      end;
end;

function TTestEngine.GetWantToFinishTest: BOOLEAN;
begin
  //Chance to finish the test if incomplete
  Result := False;

  if TestList.TestMode = tmPatient then Exit; //b116

  if not TestEngine.EndOfTest and
    (length(TestEngine.ChoicesMade) > 0) then
    if MsgDlg('This ' + MHTest.Code + ' is not completed yet.'
      + CRLF + '' + CRLF + 'Do you want to finish it now?',
      DlgCaption, mtWarning, [mbYes, mbNo], 0) = mrYes then
      Result := True;
end;

end.

