unit uMHA_Registry;
{- Designed by Roger Schultz, Ph.D.
   Mental Health Strategic Health Care Group, Informatics Section
   Revision Date: 7/2001
   Description: This unit saves default information in the Window's registry. -}

interface

uses graphics;

type
  TMHA_Registry = class(TObject)
  private
    FUseSpeedTab,
    FUseDefaultWindow: boolean; // Keyboard entry moves to next item automatically
    FLastProgram, // Last program is default for combo box, item G3
    // Window points of reference
    FMainTop,
      FMainLeft,
      FMainWidth,
      FMainHeight,
      FMainTabOrder,
    // Splitters
    FASIGeneral,
    FASIMedical, // location of split set by user
      FASIEmploy, // default for all splitters is 80
      FASIDrug, // saved in windows registry when fASI_2001_00 is closed
      FASILegal,
      FASIFamily,
      FASISocial,
      FASIPsych: integer;
    FWorkDrive : string;
    FHighlightForeground,
      FHighlightBackground: TColor;
    FShowAllHints: boolean;

    function CheckForFilesAndValues: boolean;
    function ColorStrToTColor(ColorIn: string): TColor;
    function TColorToColorStr(UserColor: TColor): string;
  protected
  public
    procedure SaveRegistry;
    function ReadRegistry: boolean;
    procedure UseDefaultRegistry;
    // Main desktop settings
    property MainTop: Integer read FMainTop write FMainTop;
    property MainLeft: Integer read FMainLeft write FMainLeft;
    property MainWidth: Integer read FMainWidth write FMainWidth;
    property MainHeight: Integer read FMainHeight write FMainHeight;
    property MainTabOrder: Integer read FMainTabOrder write FMainTabOrder;
    // Splitters
    property ASIGeneral: Integer read FASIGeneral write FASIGeneral;
    property ASIMedical: Integer read FASIMedical write FASIMedical;
    property ASIEmploy: Integer read FASIEmploy write FASIEmploy;
    property ASIDrug: Integer read FASIDrug write FASIDrug;
    property ASILegal: Integer read FASILegal write FASILegal;
    property ASIFamily: Integer read FASIFamily write FASIFamily;
    property ASISocial: Integer read FASISocial write FASISocial;
    property ASIPsych: Integer read FASIPsych write FASIPsych;
    property LastProgram: Integer read FLastProgram write FLastProgram;
    property UseSpeedTab: boolean read FUseSpeedTab write FUseSpeedTab;
    property UseDefaultWindow: boolean read FUseDefaultWindow write FUseDefaultWindow;
    property ShowAllHints: boolean read FShowAllHints write FShowAllHints;
    property HighlightForeground: TColor read FHighlightForeground write FHighlightForeground;
    property HighlightBackground: TColor read FHighlightBackground write FHighlightBackground;
  published
  end;

implementation

uses
  Controls, Dialogs, Forms,
  Registry, uMHA_MsgDlg,
  StDateSt, StDate, StUtils, SysUtils,
  Windows, WinProcs, WinTypes;

const
  WindowsRegistry: string = 'SOFTWARE\VISTA\YS\MHA\';

function TMHA_Registry.ReadRegistry: boolean;
var
  ReadReg: TRegistry;
begin
  Result := False;
  ReadReg := TRegistry.Create;
  try
    with ReadReg do
    begin
      RootKey := HKEY_CURRENT_USER;  // better for Thin client applications
      if OpenKey(WindowsRegistry, true) then
      begin
        if not ValueExists('MainLeft') then
        begin
          CloseKey;
          Free;
          UseDefaultRegistry;
          SaveRegistry;
          Result := true;
          exit;
        end;
           // location of Main form on user's screen
        FMainLeft := ReadInteger('MainLeft');
        FMainTop := ReadInteger('MainTop');
        FMainHeight := ReadInteger('MainHeight');
        FMainWidth := ReadInteger('MainWidth');
        FMainTabOrder := ReadInteger('MainTabOrder');
        // Location of splitter, user defined
        FASIGeneral := ReadInteger('ASIGeneral');
        FASIMedical := ReadInteger('ASIMedical');
        FASIEmploy := ReadInteger('ASIEmploy');
        FASIDrug := ReadInteger('ASIDrug');
        FASILegal := ReadInteger('ASILegal');
        FASIFamily := ReadInteger('ASIFamily');
        FASISocial := ReadInteger('ASISocial');
        FASIPsych := ReadInteger('ASIPsych');
          // Item-index of G3, list of programs
        FLastProgram := ReadInteger('LastProgram');
          // Toggle user's preference
        FUseSpeedTab := ReadBool('UseSpeedTab');
        FUseDefaultWindow := ReadBool('UseDefaultWindow');
        FShowAllHints := ReadBool('ShowAllHints');
        FHighlightForeground := ColorStrToTColor(ReadString('UserColorForeground'));
        FHighlightBackground := ColorStrToTColor(ReadString('UserColorBackground'));
      end;
      CloseKey;
      Free;
    end; {- with ReadReg -}
  except
    on E: exception do
    begin
        // Load default values so program will load
      UseDefaultRegistry;
    end;
  end; {- try -}

  if not CheckForFilesAndValues then exit;
  result := true;
end;

procedure TMHA_Registry.SaveRegistry;
// Saved when user closes fASI
var
  SaveReg: TRegistry;
begin
  SaveReg := TRegistry.Create;
  try
    with SaveReg do
    begin
      RootKey := HKEY_CURRENT_USER; // better for Thin client applications
      if OpenKey(WindowsRegistry, true) then
      begin
        WriteInteger('MainLeft', FMainLeft);
        WriteInteger('MainTop', FMainTop);
        WriteInteger('MainHeight', FMainHeight);
        WriteInteger('MainWidth', FMainWidth);
        WriteInteger('MainTabOrder', FMainTabOrder);
        WriteInteger('ASIGeneral', FASIGeneral);
        WriteInteger('ASIMedical', FASIMedical);
        WriteInteger('ASIEmploy', FASIEmploy);
        WriteInteger('ASIDrug', FASIDrug);
        WriteInteger('ASILegal', FASILegal);
        WriteInteger('ASIFamily', FASIFamily);
        WriteInteger('ASISocial', FASISocial);
        WriteInteger('ASIPsych', FASIPsych);
        WriteInteger('LastProgram', FLastProgram);
        WriteBool('UseSpeedTab', FUseSpeedTab);
        WriteBool('UseDefaultWindow', FUseDefaultWindow);
        WriteBool('ShowAllHints', FShowAllHints);
        //WriteString('WorkDrive', FWorkDrive);
        WriteString('UserColorForeground', TColorToColorStr(HighlightForeground {TColor}));
        WriteString('UserColorBackground', TColorToColorStr(HighlightBackground {TColor}));
      end;
      CloseKey;
      Free;
    end; {- with SaveReg -}
  except
    on E: Exception do
    begin
      exit;
    end;
  end; {- try-}
end;

procedure TMHA_Registry.UseDefaultRegistry;
begin
  FMainTop := 10;
  FMainLeft := 15;
  FMainWidth := 638;
  FMainHeight := 400;
  FMainTabOrder := 0;
  FASIGeneral := 80;
  FASIMedical := 80;
  FASIEmploy := 80;
  FASIDrug := 80;
  FASILegal := 80;
  FASIFamily := 80;
  FASISocial := 80;
  FASIPsych := 80;
  FLastProgram := 1;
  FUseSpeedTab := true;
  FHighlightForeground := clBlack;
  FHighlightBackground := clYellow;
  FUseDefaultWindow := True;
  FShowAllHints := True;
end;

function TMHA_Registry.CheckForFilesAndValues: boolean;
// This will ensure that all the proper files are present.

  function CheckTop(TopIn: integer): integer;
  begin
    Result := TopIn;
    if TopIn <= 0 then Result := 15;
  end;

  function CheckLeft(LeftIn: integer): integer;
  begin
    Result := LeftIn;
    if LeftIn <= 0 then Result := 15;
    if LeftIn > Screen.Width then
      Result := Screen.Width div 6;
  end;

  function CheckHeight(HeightIn: integer): integer;
  begin
    Result := HeightIn;
    if HeightIn < 480 then Result := 480;
    if HeightIn > Screen.Height then
      Result := Screen.Height - 20;
  end;

  function CheckWidth(WidthIn: integer): integer;
  begin
    Result := WidthIn;
    if WidthIn < 640 then Result := 640;
    if WidthIn > Screen.Width then
      Result := Screen.Width - 20;
  end;

  function CheckPercent(PercentIn: integer): integer;
  begin
    Result := PercentIn;
    if PercentIn < 10 then Result := 80;
    if PercentIn > 99 then Result := 80;
  end;

begin
  Result := false;
  FMainLeft := CheckLeft(FMainLeft);
  FMainTop := CheckTop(FMainTop);
  FMainHeight := CheckHeight(FMainHeight);
  FMainWidth := CheckWidth(FMainWidth);
  FASIGeneral := CheckPercent(FASIGeneral);
  FASIMedical := CheckPercent(FASIMedical);
  FASIEmploy := CheckPercent(FASIEmploy);
  FASIDrug := CheckPercent(FASIDrug);
  FASILegal := CheckPercent(FASILegal);
  FASIFamily := CheckPercent(FASIFamily);
  FASISocial := CheckPercent(FASISocial);
  FASIPsych := CheckPercent(FASIPsych);

  if ((FLastProgram < 0) or (FLastProgram > 20)) then
    FLastProgram := 3;
  Result := true;
end;

function TMHA_Registry.ColorStrToTColor(ColorIn: string): TColor;
// Windows registry cannot save TColor, this converts string to TColor.
begin
  Result := clBlack;
  if ColorIn = 'Yellow' then Result := clYellow;
  if ColorIn = 'Blue' then Result := clBlue;
  if ColorIn = 'Black' then Result := clBlack;
  if ColorIn = 'Gray' then Result := clGray;
  if ColorIn = 'DkGray' then Result := clDkGray;
  if ColorIn = 'Green' then Result := clGreen;
  if ColorIn = 'Lime' then Result := clLime;
  if ColorIn = 'Maroon' then Result := clMaroon;
  if ColorIn = 'Navy' then Result := clNavy;
  if ColorIn = 'Olive' then Result := clOlive;
  if ColorIn = 'Purple' then Result := clPurple;
  if ColorIn = 'Red' then Result := clRed;
  if ColorIn = 'Silver' then Result := clSilver;
  if ColorIn = 'Teal' then Result := clTeal;
  if ColorIn = 'Aqua' then Result := clAqua;
  if ColorIn = 'White' then Result := clWhite;
end;

function TMHA_Registry.TColorToColorStr(UserColor: TColor): string;
// Windows registry cannot save TColor, this converts it to a string.
begin
  if UserColor = clYellow then Result := 'Yellow';
  if UserColor = clBlue then Result := 'Blue';
  if UserColor = clBlack then Result := 'Black';
  if UserColor = clGray then Result := 'Gray';
  if UserColor = clDkGray then Result := 'DkGray';
  if UserColor = clGreen then Result := 'Green';
  if UserColor = clLime then Result := 'Lime';
  if UserColor = clMaroon then Result := 'Maroon';
  if UserColor = clNavy then Result := 'Navy';
  if UserColor = clOlive then Result := 'Olive';
  if UserColor = clPurple then Result := 'Purple';
  if UserColor = clRed then Result := 'Red';
  if UserColor = clSilver then Result := 'Silver';
  if UserColor = clTeal then Result := 'Teal';
  if UserColor = clAqua then Result := 'Aqua';
  if UserColor = clWhite then Result := 'White';
end;

end.

