{***************************************************************

 UNIT NAME: uMHA_SessionControl
 PACKAGE: Mental Health
 WARNING:
 DATE CREATED: Sep, 2000
 SITE NAME: Salt Lake City Mental Health Informatics Section
 DEVELOPERS: CANNON/FELDMAN
 DESCRIPTION:
 NOTE: This unit requires Mental Health patch 66, or greater, in
       order to run.

 HISTORY  :
 021502   Added TUserSession class.BF

 Required third-party Delphi components and libraries:

****************************************************************}
unit uMHA_SessionControl;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls;

type
  TPatientSubSession = class(TObject)
  private
    FListOfIncompleteTests: TStrings;
    FListOfPreviousGAFs: TStrings;
    FListOfPreviousTests: TStrings;
    FListOfPreviousASIs: TStrings;
    FScaleScores: TStrings;
    FASIReport: TStrings;
    FTestReport: TStrings;
    FMCMI2_PatientTraits: string;
    FPatientIsDead: BOOLEAN;
    FDOD: TDate;

  protected

  public
    constructor Create;
    destructor Destroy; override;
    function DeceasedPatientDataEntryOK(Instrument: string): BOOLEAN;
    procedure ResetDateOfDeath;
    property ListOfIncompleteTests: TStrings read FListOfIncompleteTests write
    FListOfIncompleteTests;
    property ListOfPreviousGAFs: TStrings read FListOfPreviousGAFs write
    FListOfPreviousGAFs;
    property ListOfPreviousTests: TStrings read FListOfPreviousTests write
    FListOfPreviousTests;
    property ListOfPreviousASIs: TStrings read FListOfPreviousASIs write
    FListOfPreviousASIs;
    property ScaleScores: TStrings read FScaleScores write FScaleScores;
    property ASIReport: TStrings read FASIReport write FASIReport;
    property TestReport: TStrings read FTestReport write FTestReport;
    property MCMI2_PatientTraits: string read FMCMI2_PatientTraits write
    FMCMI2_PatientTraits;
    property PatientIsDead: BOOLEAN read FPatientIsDead write FPatientIsDead;
    property DOD: TDate read FDOD write FDOD;

  published

  end;

type
  TUserSession = class(TObject)
  private
    FTempFolderName: string;
    FIsCPRSTool: BOOLEAN;
    FCPRSFMREF: string;
    FCPRSPatientDFN: string;
    FCPRSFDUZ: string;
    FCPRSBrokerListenerPort: string;
    FCPRSBrokerServer: string;
    FCPRSHandle: THandle;
{03/19}        FCPRSPatientChanged: BOOLEAN;
    function GetTempFolderName: string;
    procedure SetTempFolderName;
  protected
  public
    property TempFolderName: string read GetTempFolderName;

    { BEGIN CPRS Tools menu}
    property IsCPRSTool: BOOLEAN read FIsCPRSTool write FIsCPRSTool;
    property CPRSFMREF: string read FCPRSFMREF write FCPRSFMREF;
    property CPRSPatientDFN: string read FCPRSPatientDFN write FCPRSPatientDFN;
    property CPRSFDUZ: string read FCPRSFDUZ write FCPRSFDUZ;
    property CPRSBrokerListenerPort: string read FCPRSBrokerListenerPort write FCPRSBrokerListenerPort;
    property CPRSBrokerServer: string read FCPRSBrokerServer write FCPRSBrokerServer;
    property CPRSHandle: THandle read FCPRSHandle write FCPRSHandle;
    property CPRSPatientChanged: BOOLEAN read FCPRSPatientChanged write FCPRSPatientChanged;
    { END CPRS Tools menu}

  published
  end; { UserSession }

implementation

uses fMHA_Main, TRPCB, MFUNStr, ESBDates,
  uCore, rCore,
  fMHA_MsgDlg, // for message dialogs
  ORNet, rMHA_VISTA, Dialogs, ORfn;

//******************************************************************************
// TPatientSubSession.Create [X]USED [X]DONE [ ]TESTED
// PURPOSE: Init stringlists
// AUTHOR: R. FELDMAN // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 04/27/01
//******************************************************************************

constructor TPatientSubSession.Create;
begin
  inherited Create;
  FListOfPreviousGAFs := TStringList.Create;
  FListOfPreviousTests := TStringList.Create;
  FListOfIncompleteTests := TStringList.Create;
  FListOfPreviousASIs := TStringList.Create;
  FScaleScores := TStringList.Create;
  FASIReport := TStringList.Create;
  FTestReport := TStringList.Create;
end;

//******************************************************************************
// TPatientSubSession.Destroy [X]USED [X]DONE [ ]TESTED
// PURPOSE: Destroy Stringlists
// AUTHOR: R. FELDMAN // REVIEWER: Cannon
// LAST MODIFICATION/REVIEW DATE: 04/27/01
//******************************************************************************

destructor TPatientSubSession.Destroy;
begin
  FTestReport.Free;
  FASIReport.Free;
  FScaleScores.Free;
  FListOfPreviousASIs.Free;
  FListOfIncompleteTests.Free;
  FListOfPreviousTests.Free;
  FListOfPreviousGAFs.Free;
  inherited Destroy;
end;

procedure TPatientSubSession.ResetDateOfDeath;
var
  tmpDOD: Double;
begin

  tmpDOD := DateOfDeath(Patient.DFN);
  PatientIsDead := (tmpDOD > 0);
  if PatientIsDead then
    DOD := FMDateTimeToDateTime(tmpDOD);

end;

function TPatientSubSession.DeceasedPatientDataEntryOK(Instrument: string): BOOLEAN;
begin
  result := (MsgDlg(Patient.Name + ' died on ' + DateToStr(DOD) + '.' + CRLF + CRLF
    + 'Are you sure you want to enter a new ' + Instrument + ' for this deceased patient?',
    DlgCaption, mtConfirmation, [mbYes, mbNo], 0) = mrYes);
end;

//******************************************************************************
// UserSession.SetTempFolderName [ ]USED [ ]DONE [ ]TESTED
// PURPOSE:
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/15/2002
//******************************************************************************

procedure TUserSession.SetTempFolderName;
var
//  WinTempPath: string;
//  WinPath: string;
  buf: array[0..MAX_PATH] of Char;
begin
  try
    GetTempPath(Sizeof(buf), buf); // i.e. c:\temp
    FTempFolderName := buf;

    if FTempFolderName = '' then
    begin
      GetWindowsDirectory(Buf, Sizeof(Buf)); // i.e. C:\windows
      FTempFolderName := StrPas(Buf);
    end;
  except
    FTempFolderName := 'c:\';
  end;

end;

//******************************************************************************
// UserSession.GetTempFolderName [ ]USED [ ]DONE [ ]TESTED
// PURPOSE:
// AUTHOR: FELDMAN // REVIEWER:
// LAST MODIFICATION/REVIEW DATE: 02/15/2002
//******************************************************************************

function TUserSession.GetTempFolderName: string;
begin
  if FTempFolderName = '' then
    setTempFolderName;

  result := FTempFolderName;
end;

end.

