{***************************************************************

 UNIT NAME: uMain (SecureDesktop)
 PACKAGE: Mental Health
 WARNING:
 DATE CREATED: Sep, 2001
 SITE NAME: Salt Lake City Mental Health Informatics Section
 DEVELOPERS: Feldman
 DESCRIPTION:
 NOTE: This unit requires Mental Health patch 67, or greater, in
       order to run.

 HISTORY  :

 Required third-party Delphi components and libraries:

****************************************************************

 NOTES:
    Part of this unit is based on the work of Joe Hecht, who posted this
    code on the Internet as sample code to accomplish these tasks. He
    graciously granted permission for our using it, and modifying it for
    this project.

    Portions Copyright by Joe C. Hecht - All rights reserved.
    Used by permission.
    email: joehecht@code4sale.com
}

unit uMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
    StdCtrls,
  ExtCtrls, Menus, abfComponents;

type
  TfrmSDMain = class(TForm)
    Button1: TButton;
    Memo1: TMemo;
    SecurityNotice: TStaticText;
    PopupMenu1: TPopupMenu;
    Exit1: TMenuItem;
    Label1: TLabel;
    btnStop: TButton;
    Timer1: TTimer;
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure EnableSecurity(Sender: TObject);
    procedure DisableSecurity(Sender: TObject);
    procedure GetWins;
    procedure FormShow(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
  private
    { Private declarations }
//    TWatchedAppHasShutDown: Boolean;
    FViolationCount: ShortInt;
    //NOTE: Changed this to Byte, to resolve warning.
    FCurrentKeystrokes: Byte;
  public
    { Public declarations }
  end;

var
  frmSDMain: TfrmSDMain;
  WindowList: TList;

implementation

{$R *.DFM}

{#Todo1 toggle these, before releasing product}
{$DEFINE RELEASE_VER}
{.$DEFINE DEBUG_VER}

{Functions prototypes for the hook dll}
type
  TGetHookRecPointer = function: pointer stdcall;

type
  TStartKeyBoardHook = procedure stdcall;

type
  TStopKeyBoardHook = procedure stdcall;

  {The record type filled in by the hook dll}
type
  THookRec = packed record
    TheHookHandle: HHOOK;
    TheAppWinHandle: HWND;
    TheCtrlWinHandle: HWND;
    TheKeyCount: DWORD;
    IsStartKeyDown: Boolean;
  end;

  {A pointer type to the hook record}
type
  PHookRec = ^THookRec;

var
  hHookLib: THANDLE; {A handle to the hook dll}
  GetHookRecPointer: TGetHookRecPointer; {Function pointer}
  StartKeyBoardHook: TStartKeyBoardHook; {Function pointer}
  StopKeyBoardHook: TStopKeyBoardHook; {Function pointer}
  LibLoadSuccess: bool; {If the hook lib was successfully loaded}
  lpHookRec: PHookRec; {A pointer to the hook record}
  EnterKeyCount: DWORD; {An internal count of the Enter Key}

function GetWindows(Handle: HWND; Info: Pointer): BOOL; stdcall;
begin
  Result := True;
  WindowList.Add(Pointer(Handle));
end;

procedure TfrmSDMain.GetWins;
var
  TopWindow, CurrentWindow: HWND;
  Dest: array[0..80] of char;
  ClassName: array[0..80] of char;
  i: Integer;
begin
  try
    WindowList := TList.Create;
    TopWindow := Handle;
    EnumWindows(@GetWindows, Longint(@TopWindow));
    CurrentWindow := TopWindow;
    for i := 0 to WindowList.Count - 1 do
    begin
      CurrentWindow := GetNextWindow(CurrentWindow, GW_HWNDNEXT);
      GetWindowText(CurrentWindow, Dest, sizeof(Dest) - 1);
      GetClassName(CurrentWindow, ClassName, sizeof(ClassName) - 1);

      if Windows.IsWindowVisible(CurrentWindow) then
        if StrLen(Dest) > 0 then
        begin
          if (string(Dest) <> 'SecureDesktop') and
{$IFDEF DEBUG_VER}
          (string(Dest) <> 'Delphi 5') and
            (string(Dest) <> 'Delphi 5 - YS50171_MHA_SD') and
            (string(Dest) <> 'Program Manager') and
{$ENDIF}
          (string(Dest) <> 'Mental Health Assistant') and
            (string(ClassName) <> 'TfrmTestEntry') //and
          {//            (string(Dest) <> 'Addiction Severity Index- Multimedia Version, VistA Interface')} then
          begin
            PostMessage(CurrentWindow, WM_SYSCOMMAND, SC_CLOSE, 0);

            //for debug            Memo1.Lines.Add(Dest + ' = ' + ClassName + ' - ' + IntToStr(CurrentWindow) +
            //              ' - ' + IntToStr(Windows.GetParent(CurrentWindow)));
          end;

        end;
    end;
  finally
    WindowList.Free;
  end;
end;

procedure TfrmSDMain.Timer1Timer(Sender: TObject); //200 msec timer event
var
  wndTestEntry,
    //    wndASIMV4Main,
  wndForeground: HWND;
  i: integer;
begin

{$IFDEF DEBUG_VER}
  {#TODO3 Remove label to prevent flicker?}
  Label1.Caption := 'Number of keystrokes: ' + IntToStr(lpHookRec^.TheKeyCount);
{$ENDIF}

{$IFDEF RELEASE_VER}
  //Detect if a new keystroke was pressed. If so, trigger screensaver.
  if FCurrentKeystrokes <> lpHookRec^.TheKeyCount then
  begin
    //Trigger the screensaver, which should be password-protected.
    PostMessage(lpHookRec^.TheCtrlWinHandle, WM_SYSCOMMAND, SC_SCREENSAVE, 0);
    FCurrentKeystrokes := lpHookRec^.TheKeyCount;
  end;
{$ENDIF}

  {#Todo2 review number of allowed keystrokes before shutdown is done}
  if lpHookRec^.TheKeyCount >= 5 then
  begin
    Close;
    exit;
  end;

  {#TODO1 Review this rule of 5}
  if FViolationCount > 5 then
  begin
    close;
    exit;
  end;

  {#TODO2 REVIEW: Force minimize.
  I can reinstate this if I subtract the two keystrokes that it causes, from the
  keystrokeCount.}
    //Force Minimize all windows
  //+?  keybd_event(VK_LWIN, MapvirtualKey(VK_LWIN, 0), 0, 0);
  //  keybd_event(Ord('M'), MapvirtualKey(Ord('M'), 0), 0, 0);
  //  keybd_event(Ord('M'), MapvirtualKey(Ord('M'), 0), KEYEVENTF_KEYUP, 0);
  //  keybd_event(VK_LWIN, MapvirtualKey(VK_LWIN, 0), KEYEVENTF_KEYUP, 0);

    //Look for an open task manager window. If found, shut it down.
  i := FindWindow(nil, 'Windows NT Task Manager');
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Look for an open W2K task manager window. If found, shut it down.
  i := FindWindow(nil, 'Windows Task Manager');
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Look for an open task manager security window. If found, shut it down.
  i := FindWindow(nil, 'Windows NT Security');
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Look for an open W2K task manager security window. If found, shut it down.
  i := FindWindow(nil, 'Windows Security');
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Look for an open W2K task manager Shut Down window. If found, shut it down.
  i := FindWindow(nil, 'Shut Down Windows');
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Look for an open W2K task manager Change Password window. If found, shut it down.
  i := FindWindow(nil, 'Change Password');
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Look for an open Run window. If found, shut it down.
  i := FindWindow(nil, 'Run');
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Look for an open Find window. If found, shut it down.
  i := FindWindow(nil, 'Find: All Files');
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Look for an open W2K task manager Log Off Windows window. If found, shut it down.
  i := FindWindow(nil, 'Log Off Windows');
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Look for an open Explorer window. If found, shut it down.
  i := FindWindow('ExploreWClass', nil);
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Look for an open Explorer window. If found, shut it down.
  i := FindWindow('MS_WINHELP', nil);
  if not (i = 0) then
  begin
    PostMessage(i, WM_CLOSE, 0, 0);
    FViolationCount := FViolationCount + 1;
  end;

  //Get handles to key applications (windows) that we wish to monitor.
  {#Todo3 use an if wndTestEntry = 0 then get the new handle, so we don't search for it everytime}
  wndTestEntry := FindWindow('TfrmTestEntry', nil); //ThunderRT6Main
  //  wndASIMV4Main := FindWindow('ThunderRT6Main', nil); //ASIMV4 Main form
  wndForeground := GetForegroundWindow;

  if (wndTestEntry > 0) {or (wndASIMV4Main > 0)} then
    //Monitored applications are still active... so continue.

  begin
    if wndForeground <> wndTestEntry then
      //Application is not the in the foreground (topmost)
      SetForegroundWindow(wndTestEntry)
        //testing     SetForegroundWindow(Application.Handle);
        //    TWatchedAppHasShutDown := False;
  end
  else
  begin
{$IFDEF RELEASE_VER}
    close;
    exit;
{$ENDIF}
  end;

end;

procedure EnableTaskBar(Status: Boolean);
begin
  if Status = True then
    EnableWindow(FindWindow
      ('Shell_TrayWnd', nil), TRUE)
  else
    EnableWindow(FindWindow
      ('Shell_TrayWnd', nil), FALSE);
end;

procedure TfrmSDMain.EnableSecurity(Sender: TObject);
var
  h: hwnd;
begin
  //Hide all Desktop Icons
  ShowWindow(FindWindowEx(FindWindow('Progman', nil), 0, 'ShellDll_DefView',
    nil), SW_HIDE);

  {#todo3 replace this with the disable method only. Don't need to enable taskbar after start.}
    //Disable taskbar
  EnableTaskBar(False);

  //disable the start button
  h := FindWindowEx(FindWindow('Shell_TrayWnd', nil), 0, 'Button', nil);
  ShowWindow(h, 0);
  Windows.SetParent(h, 0);

  //Look for a Microsoft Office Shortcut Bar. If found, shut it down.
  h := FindWindow(nil, 'Microsoft Office Shortcut Bar');
  if not (h = 0) then
    PostMessage(h, WM_CLOSE, 0, 0);

  FViolationCount := 0;

end;

procedure TfrmSDMain.DisableSecurity(Sender: TObject);
var
  h: hwnd;
  TaskWindow: hwnd;
begin
  {Show Desktop Icons}
  ShowWindow(FindWindowEx(FindWindow('Progman', nil), 0, 'ShellDll_DefView',
    nil), SW_SHOW);

  //Enable the start button
  h := FindWindowEx(GetDesktopWindow, 0, 'Button', nil);
  TaskWindow := FindWindow('Shell_TrayWnd', nil);
  Windows.SetParent(h, TaskWindow);
  ShowWindow(h, 1);

  //enable taskbar
  EnableTaskBar(true);
end;

procedure TfrmSDMain.FormCreate(Sender: TObject);
begin
  {Set our initial variables}
  Timer1.Enabled := FALSE;

  Timer1.Interval := 200;
  EnterKeyCount := 0;
  lpHookRec := nil;
  LibLoadSuccess := FALSE;
  @GetHookRecPointer := nil;
  @StartKeyBoardHook := nil;
  @StopKeyBoardHook := nil;
  {Try to load the hook dll}
  hHookLib := LoadLibrary('YS_MHA_KH.DLL');
  {If the hook dll was loaded successfully}
  if hHookLib <> 0 then
  begin
    {Get the function addresses}
    @GetHookRecPointer :=
      GetProcAddress(hHookLib, 'GETHOOKRECPOINTER');
    @StartKeyBoardHook :=
      GetProcAddress(hHookLib, 'STARTKEYBOARDHOOK');
    @StopKeyBoardHook :=
      GetProcAddress(hHookLib, 'STOPKEYBOARDHOOK');
    {Did we find all the functions we need?}
    if ((@GetHookRecPointer <> nil) and
      (@StartKeyBoardHook <> nil) and
      (@StopKeyBoardHook <> nil)) then
    begin
      LibLoadSuccess := TRUE;
      {Get a pointer to the hook record}
      lpHookRec := GetHookRecPointer;
      {Were we successfull in getting a ponter to the hook record}
      if (lpHookRec <> nil) then
      begin
        {Fill in our portion of the hook record}
        lpHookRec^.TheHookHandle := 0;
        lpHookRec^.TheCtrlWinHandle := Button1.Handle;
        lpHookRec^.TheKeyCount := 0;
        lpHookRec^.IsStartKeyDown := false;
        {Start the keyboard hook}
        StartKeyBoardHook;
        {Start the timer if the hook was successfully set}
        if (lpHookRec^.TheHookHandle <> 0) then
        begin
          Timer1.Enabled := TRUE;
        end;
      end;
    end
    else
    begin
      {We failed to find all the functions we need}
      FreeLibrary(hHookLib);
      hHookLib := 0;
      @GetHookRecPointer := nil;
      @StartKeyBoardHook := nil;
      @StopKeyBoardHook := nil;
    end;
  end;

  EnableSecurity(self);

{$IFDEF DEBUG_VER}
  btnStop.Visible := True;
  Label1.Visible := True;
  frmSDMain.PopupMenu := PopupMenu1;
  frmSDMain.Height := 333;
  //+  Timer1.Enabled := FALSE;
{$ENDIF}
{$IFDEF RELEASE_VER}
  btnStop.Visible := False;
  Label1.Visible := False;
  frmSDMain.PopupMenu := nil;
  //  frmSDMain.Height := 78;
{$ENDIF}

end;

procedure TfrmSDMain.FormDestroy(Sender: TObject);
begin
  {Did we load the dll successfully?}
  if (LibLoadSuccess = TRUE) then
  begin
    {Did we sucessfully get a pointer to the hook record?}
    if (lpHookRec <> nil) then
    begin
      {Did the hook get set?}
      if (lpHookRec^.TheHookHandle <> 0) then
      begin
        Timer1.Enabled := FALSE;
        StopKeyBoardHook;
      end;
    end;
    {Free the hook dll}
    FreeLibrary(hHookLib);
  end;

{$IFDEF DEBUG_VER}
  DisableSecurity(Self);
{$ENDIF}

  //Logout the current NT user, forcing all apps to close
{$IFDEF RELEASE_VER}
  ExitWindowsEx(EWX_LOGOFF + EWX_FORCE, 0);
{$ENDIF}
end;

procedure TfrmSDMain.FormShow(Sender: TObject);
begin
{$IFDEF RELEASE_VER}
  GetWins;
{$ENDIF}
end;

procedure TfrmSDMain.btnStopClick(Sender: TObject);
begin
  Close;
end;

end.

